@@ -1,3 +1,5 @@
+#include <config.h>
+#ifdef ARTIST_BROWSER
 /*
  *Copyright (C) 2004-2007 Qball Cow <qball@sarine.nl>
  *
@@ -32,6 +34,7 @@
 #include "playlist3-artist-browser.h"
 #include "config1.h"
 #include "TreeSearchWidget.h"
+#include <libmpd/libmpd-internal.h>
 
 static void pl3_artist_browser_add(GtkWidget *cat_tree);
 static void pl3_artist_browser_fill_tree(GtkWidget *, GtkTreeIter *);
@@ -50,8 +53,10 @@ static int pl3_artist_browser_playlist_key_press(GtkWidget *tree, GdkEventKey *e
 static void pl3_artist_browser_connection_changed(MpdObj *mi, int connect, gpointer data);
 static int pl3_artist_browser_key_press_event(GtkWidget *mw, GdkEventKey *event, int type);
 static void pl3_artist_browser_status_changed(MpdObj *mi,ChangedStatusType what, void *data);
+static MpdData *pl3_artist_browser_sort_tracks(MpdData *);
 
 static void pl3_artist_browser_reupdate(void);
+static void pl3_artist_browser_save_myself(void);
 
 enum{
 	PL3_AB_ARTIST,
@@ -63,6 +68,40 @@ enum{
 	PL3_AB_ROWS
 };
 extern GladeXML *pl3_xml;
+GtkTreeRowReference *pl3_ab_tree_ref = NULL;
+/**
+ * Helper functions 
+ */
+static void pl3_artist_browser_set_enabled(int enabled)
+{
+	cfg_set_single_value_as_int(config, "artist-browser", "enable", enabled);
+	if (enabled)
+	{
+		if(pl3_ab_tree_ref == NULL)
+		{
+			pl3_artist_browser_add(GTK_WIDGET(playlist3_get_category_tree_view()));
+		}
+	}
+	else if (pl3_ab_tree_ref)
+	{
+		GtkTreePath *path = gtk_tree_row_reference_get_path(pl3_ab_tree_ref);
+		if (path){
+			GtkTreeIter iter;
+			if (gtk_tree_model_get_iter(GTK_TREE_MODEL(playlist3_get_category_tree_store()), &iter, path)){
+				gtk_tree_store_remove(playlist3_get_category_tree_store(), &iter);
+			}
+			gtk_tree_path_free(path);
+			gtk_tree_row_reference_free(pl3_ab_tree_ref);
+			pl3_ab_tree_ref = NULL;
+		}                                                                                                  	
+	}                                                                                                      	
+	pl3_update_go_menu();
+}
+
+static int pl3_artist_browser_get_enabled(void)
+{
+	return	cfg_get_single_value_as_int_with_default(config, "artist-browser", "enable", TRUE); 
+}
 
 /**
  * Plugin structure
@@ -80,20 +119,15 @@ gmpcPlBrowserPlugin artist_browser_gbp = {
 };
 
 gmpcPlugin artist_browser_plug = {
-	"Artist Browser",
-	{1,1,1},
-	GMPC_PLUGIN_PL_BROWSER,
-	0,
-	NULL,			                /* path*/
-	NULL,			                /* init */
-        NULL,                                   /* Destroy */
-	&artist_browser_gbp,		        /* Browser */
-	pl3_artist_browser_status_changed,	/* status changed */
-	pl3_artist_browser_connection_changed, 	/* connection changed */
-	NULL,		                        /* Preferences */
-	NULL,			                /* MetaData */
-	NULL,			                /* Get enable */
-	NULL			                /* Set Enable */
+	.name 					= "Artist Browser",
+	.version 				= {1,1,1},
+	.plugin_type 			= GMPC_PLUGIN_PL_BROWSER,
+	.browser 				= &artist_browser_gbp,		        /* Browser */
+	.mpd_status_changed		= pl3_artist_browser_status_changed,	/* status changed */
+	.mpd_connection_changed = pl3_artist_browser_connection_changed, 	/* connection changed */
+	.get_enabled 			= pl3_artist_browser_get_enabled,
+	.set_enabled 			= pl3_artist_browser_set_enabled,
+	.save_yourself			= pl3_artist_browser_save_myself
 };
 
 /* internal */
@@ -104,16 +138,24 @@ GtkWidget *pl3_ab_tree_search = NULL;
 
 
 
-GtkTreeRowReference *pl3_ab_tree_ref = NULL;
+
 
 static int pl3_artist_browser_button_press_event(GtkTreeView *tree, GdkEventButton *event)
 {
-	GtkTreeSelection *sel = gtk_tree_view_get_selection(tree);
-	if(event->button != 3 || gtk_tree_selection_count_selected_rows(sel) < 2|| !mpd_check_connected(connection))
-	{
-		return FALSE;
+	GtkTreePath *path = NULL;
+	if(event->button == 3 &&gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(tree), event->x, event->y,&path,NULL,NULL,NULL))
+	{	
+		GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
+		if(gtk_tree_selection_path_is_selected(sel, path))
+		{
+			gtk_tree_path_free(path);
+			return TRUE;
+		}
 	}
-	return TRUE;
+	if(path) {
+		gtk_tree_path_free(path);
+	}
+	return FALSE; 
 }
 static void pl3_artist_browser_search_activate()
 {
@@ -200,10 +242,12 @@ static void pl3_artist_browser_add(GtkWidget *cat_tree)
 {
 	GtkTreePath *path;
 	GtkTreeIter iter,child;
-	gtk_tree_store_append(pl3_tree, &iter, NULL);
+	gint pos = cfg_get_single_value_as_int_with_default(config, "artist-browser","position",3);
+	playlist3_insert_browser(&iter, pos);
+//	gtk_tree_store_append(pl3_tree, &iter, NULL);
 	gtk_tree_store_set(pl3_tree, &iter, 
 			PL3_CAT_TYPE, artist_browser_plug.id,
-			PL3_CAT_TITLE, _("Browse Artists"),
+			PL3_CAT_TITLE, _("Artist Browser"),
 			PL3_CAT_INT_ID, "",
 			PL3_CAT_ICON_ID, "media-artist",
 			PL3_CAT_PROC, FALSE,
@@ -272,7 +316,60 @@ static void pl3_artist_browser_cover_art_fetched(mpd_Song *song, MetaDataResult
 
 }
 
+static MpdData *pl3_artist_browser_sort_tracks(MpdData *data)
+{
+	MpdData_real *new_list = NULL;
 
+	/* This should always be the case anyway, but just to make sure... */
+	data = mpd_data_get_first(data);
+
+	while (data) {
+		/* Insert new element 'data' into list 'new_list' */
+		MpdData_real **pos;
+		MpdData_real *data_real = (MpdData_real *)data;
+
+		data = mpd_data_get_next_real(data, FALSE);
+
+		data_real->next = NULL;
+
+		for (pos = &new_list; *pos; pos = &(*pos)->next) {
+			/* Sort on two keys. First album... */
+			int cmp = strcmp((*pos)->song->album?:"",
+					 data_real->song->album?:"");
+
+			if (cmp > 0)
+				break;
+			if (cmp < 0)
+				continue;
+
+			cmp = strcmp((*pos)->song->disc?:"",
+					 data_real->song->disc?:"");
+			if (cmp > 0)
+				break;
+			if (cmp < 0)
+				continue;
+
+			/* ... then track number */
+			if (atoi((*pos)->song->track?:"") >
+			    atoi(data_real->song->track?:""))
+				break;
+		}
+		data_real->next = *pos;
+		*pos = data_real;
+	}
+	data = (MpdData *)new_list;
+
+	if (new_list) {
+		new_list->head->first = new_list;
+
+		new_list->prev = NULL;
+		while (new_list->next) {
+			new_list->next->prev = new_list;
+			new_list = new_list->next;
+		}
+	}		
+	return data;
+}
 
 static long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
 {
@@ -334,7 +431,7 @@ static long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
           mpd_Song *song = mpd_newSong();
           song->artist = g_strdup(data->tag);
           song->album = NULL; 
-          meta_data_get_path_callback(song, META_ARTIST_ART, pl3_artist_browser_cover_art_fetched, rowref);
+          gmpc_meta_watcher_get_meta_path_callback(gmw,song, META_ARTIST_ART, pl3_artist_browser_cover_art_fetched, rowref);
           mpd_freeSong(song);	
           gtk_tree_path_free(path);
         }
@@ -389,7 +486,8 @@ static long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
 			song = mpd_newSong();                                                                     			
 			song->artist = g_strdup(artist);
 			song->album = g_strdup(data->tag);
-			meta_data_get_path_callback(song, META_ALBUM_ART, pl3_artist_browser_cover_art_fetched, rowref);
+			gmpc_meta_watcher_get_meta_path_callback(gmw,song, META_ALBUM_ART, pl3_artist_browser_cover_art_fetched, rowref);
+			//meta_data_get_path_callback(song, META_ALBUM_ART, pl3_artist_browser_cover_art_fetched, rowref);
 			mpd_freeSong(song);	
 			gtk_tree_path_free(path);
 
@@ -399,6 +497,7 @@ static long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
 
 
 		data = mpd_database_find(connection, MPD_TABLE_ARTIST, artist, TRUE);
+		data = pl3_artist_browser_sort_tracks(data);
 		/* artist is selected */
 		while(data != NULL)
 		{
@@ -465,6 +564,7 @@ static long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
 		if(pb) g_object_unref(pb);	
 		/* artist and album is selected */
 		data = mpd_database_find(connection,MPD_TABLE_ALBUM, string, TRUE);
+		data = pl3_artist_browser_sort_tracks(data);
 		while (data != NULL)
 		{
 			if(data->type == MPD_DATA_TYPE_SONG)
@@ -604,6 +704,7 @@ static void pl3_artist_browser_add_folder()
 			/* artist selected */
 			gchar *message = g_strdup_printf("Added songs from artist '%s'",artist);
 			MpdData * data = mpd_database_find(connection, MPD_TABLE_ARTIST, artist, TRUE);
+			data = pl3_artist_browser_sort_tracks(data);
 			while (data != NULL)
 			{
 				if(data->type == MPD_DATA_TYPE_SONG)
@@ -622,6 +723,7 @@ static void pl3_artist_browser_add_folder()
 			/* fetch all songs by this album and check if the artist is right. from mpd and add them to the add-list */
 			gchar *message = g_strdup_printf("Added songs from album '%s' ",title);
 			MpdData *data = mpd_database_find(connection, MPD_TABLE_ALBUM, title, TRUE);
+			data = pl3_artist_browser_sort_tracks(data);
 			while (data != NULL)
 			{
 				if(data->type == MPD_DATA_TYPE_SONG)
@@ -647,8 +749,11 @@ static void pl3_artist_browser_add_folder()
 static void pl3_artist_browser_replace_folder()
 {
 	mpd_playlist_clear(connection);
-	pl3_artist_browser_add_folder();
-	mpd_player_play(connection);
+    if(mpd_check_connected(connection))
+    {
+        pl3_artist_browser_add_folder();
+        mpd_player_play(connection);
+    }
 }
 
 static void pl3_artist_browser_category_key_press(GtkWidget *tree, GdkEventKey *event, int selected_type)
@@ -677,9 +782,9 @@ static void pl3_artist_browser_show_info()
 	{
 		GList *list = NULL;
 		list = gtk_tree_selection_get_selected_rows (selection, &model);
-		/* iterate over every row */
+
 		list = g_list_last (list);
-		do
+	//	do
 		{
 			GtkTreeIter iter;
 			char *path;
@@ -693,13 +798,18 @@ static void pl3_artist_browser_show_info()
 			{
 				mpd_Song *song = mpd_database_get_fileinfo(connection, path);
 				if(song)
-					call_id3_window_song(song);
+				{
+					info2_activate();
+					info2_fill_song_view(song);	
+					mpd_freeSong(song);
+				}
+				//	call_id3_window_song(song);
 
 			}
 			if(path)q_free(path);
 		}
-		while ((list = g_list_previous (list)) && mpd_check_connected(connection));
-		/* free list */
+	//	while ((list = g_list_previous (list)) && mpd_check_connected(connection));
+
 		g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
 		g_list_free (list);
 	}
@@ -839,7 +949,7 @@ static void pl3_artist_browser_button_release_event(GtkWidget *but, GdkEventButt
 		}
 	}
 	/* add the replace widget */
-	item = gtk_image_menu_item_new_with_label("Replace");
+	item = gtk_image_menu_item_new_with_label(_("Replace"));
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
 			gtk_image_new_from_stock(GTK_STOCK_REDO, GTK_ICON_SIZE_MENU));
 	gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), item);
@@ -850,15 +960,18 @@ static void pl3_artist_browser_button_release_event(GtkWidget *but, GdkEventButt
 	g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_artist_browser_add_selected), NULL);
 
 	gtk_widget_show_all(menu);
-	gtk_menu_popup(GTK_MENU(menu), NULL, NULL,NULL, NULL, event->button, event->time);
+	gtk_menu_popup(GTK_MENU(menu), NULL, NULL,NULL, NULL, 0, event->time);
 	return;
 }
 
 static void pl3_artist_browser_replace_selected()
 {
 	mpd_playlist_clear(connection);
-	pl3_artist_browser_add_selected();
-	mpd_player_play(connection);
+    if(mpd_check_connected(connection))
+    {
+        pl3_artist_browser_add_selected();
+        mpd_player_play(connection);
+    }
 }
 
 static void pl3_artist_browser_add_selected()
@@ -893,6 +1006,7 @@ static void pl3_artist_browser_add_selected()
 			else if (type&PL3_ENTRY_ARTIST)
 			{
 				MpdData * data = mpd_database_find(connection, MPD_TABLE_ARTIST, artist, TRUE);
+				data = pl3_artist_browser_sort_tracks(data);
 				while (data != NULL)
 				{
 					if(data->type == MPD_DATA_TYPE_SONG)
@@ -907,6 +1021,7 @@ static void pl3_artist_browser_add_selected()
 			{
 				MpdData *data = NULL;
 				data = mpd_database_find(connection, MPD_TABLE_ALBUM, album, TRUE);
+				data = pl3_artist_browser_sort_tracks(data);
 				while (data != NULL)
 				{
 					if(data->type == MPD_DATA_TYPE_SONG)
@@ -934,15 +1049,15 @@ static void pl3_artist_browser_add_selected()
 		GString *str = g_string_new("Added: ");
 		if(songs)
 		{
-			g_string_append_printf(str, "%i %s", songs, (songs > 1)?_("songs"):_("song"));
+			g_string_append_printf(str, "%i %s", songs, ngettext("song", "songs", songs));
 		}		
 		if(artists)
 		{
-			g_string_append_printf(str, "%i %s", artists, (artists > 1)?_("artists"):_("artist"));
+			g_string_append_printf(str, "%i %s", artists, ngettext("artist", "artists", artists));
 		}		
 		if(albums)
 		{
-			g_string_append_printf(str, "%i %s", albums, (albums > 1)?_("albums"):_("album"));
+			g_string_append_printf(str, "%i %s", albums, ngettext("album", "albums", albums));
 		}		
 		pl3_push_statusbar_message(str->str);
 		g_string_free(str, TRUE);
@@ -1014,7 +1129,7 @@ static int pl3_artist_browser_cat_popup(GtkWidget *menu, int type,GtkWidget *tre
 		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_artist_browser_add_folder), NULL);
 
 		/* add the replace widget */
-		item = gtk_image_menu_item_new_with_label("Replace");
+		item = gtk_image_menu_item_new_with_label(_("Replace"));
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
 				gtk_image_new_from_stock(GTK_STOCK_REDO, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
@@ -1042,6 +1157,7 @@ static void pl3_artist_browser_disconnect()
 			/* add phantom child */
 			gtk_tree_store_append(pl3_tree, &child, &iter);
 		}
+		if(path) gtk_tree_path_free(path);
 		if(pl3_ab_store) gtk_list_store_clear(pl3_ab_store);
 	}
 }
@@ -1051,9 +1167,6 @@ static void pl3_artist_browser_activate()
 	GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)
 			glade_xml_get_widget (pl3_xml, "cat_tree"));
 
-	/**
-	 * Fix this to be nnot static
-	 */	
 	GtkTreePath *path = gtk_tree_row_reference_get_path(pl3_ab_tree_ref); 
 	if(path)
 	{
@@ -1065,11 +1178,14 @@ static void pl3_artist_browser_activate()
 static int pl3_artist_browser_add_go_menu(GtkWidget *menu)
 {
 	GtkWidget *item = NULL;
+	if(!pl3_artist_browser_get_enabled())
+		return 0;
 
 	item = gtk_image_menu_item_new_with_label(_("Artist Browser"));
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
 			gtk_image_new_from_icon_name("media-artist", GTK_ICON_SIZE_MENU));
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+	gtk_widget_add_accelerator(GTK_WIDGET(item), "activate", gtk_menu_get_accel_group(GTK_MENU(menu)), GDK_F4, 0, GTK_ACCEL_VISIBLE);
 	g_signal_connect(G_OBJECT(item), "activate", 
 			G_CALLBACK(pl3_artist_browser_activate), NULL);
 
@@ -1085,12 +1201,15 @@ static void pl3_artist_browser_connection_changed(MpdObj *mi, int connect, gpoin
 }
 static int pl3_artist_browser_key_press_event(GtkWidget *mw, GdkEventKey *event, int type)
 {
+	if(!pl3_artist_browser_get_enabled())
+		return FALSE;
+/*
 	if (event->keyval == GDK_F4)
 	{
 		pl3_artist_browser_activate();
 		return TRUE;
 	}                                           	
-
+*/
 	return FALSE;
 }
 static void pl3_artist_browser_status_changed(MpdObj *mi,ChangedStatusType what, void *data)
@@ -1126,3 +1245,18 @@ static void pl3_artist_browser_reupdate(void)
 }
 
 
+static void pl3_artist_browser_save_myself(void)
+{
+	if(pl3_ab_tree_ref)
+	{
+		GtkTreePath *path = gtk_tree_row_reference_get_path(pl3_ab_tree_ref);
+		if(path)
+		{
+			gint *indices = gtk_tree_path_get_indices(path);
+			debug_printf(DEBUG_INFO,"Saving myself to position: %i\n", indices[0]);
+			cfg_set_single_value_as_int(config, "artist-browser","position",indices[0]);
+			gtk_tree_path_free(path);
+		}
+	}
+}
+#endif
