@@ -30,7 +30,6 @@
 #include "playlist3.h"
 #include "playlist3-find2-browser.h"
 #include "config1.h"
-#include "id3info.h"
 
 #include "gmpc-mpddata-model.h"
 #include "gmpc-mpddata-treeview.h"
@@ -46,12 +45,15 @@ static void pl3_find2_browser_row_activated(GtkTreeView *, GtkTreePath *);
 static int pl3_find2_browser_playlist_key_press(GtkWidget *, GdkEventKey *);
 static void pl3_find2_browser_add_selected(void);
 static gboolean pl3_find2_browser_button_release_event(GtkWidget *but, GdkEventButton *event);
+static gboolean pl3_find2_browser_button_press_event(GtkWidget *giv, GdkEventButton *event, gpointer data);
 static void pl3_find2_browser_connection_changed(MpdObj *mi, int connect, gpointer data);
 static int pl3_find2_browser_key_press_event(GtkWidget *mw, GdkEventKey *event, int type);
 static void pl3_find2_browser_status_changed(MpdObj *mi,ChangedStatusType what, void *data);
 extern GladeXML *pl3_xml;
 static void pl3_find2_combo_box_changed(GtkComboBox *cb, gpointer data);
 
+static void pl3_find2_save_myself(void);
+
 int pl3_find2_last_entry = MPD_TAG_ITEM_ANY;
 /**
  * Plugin structure
@@ -69,26 +71,21 @@ gmpcPlBrowserPlugin find2_browser_gbp = {
 };
 
 gmpcPlugin find2_browser_plug = {
-	"Database Search Browser",
-	{1,1,1},
-	GMPC_PLUGIN_PL_BROWSER,
-	0,
-	NULL,			                /* path*/
-	NULL,			                /* init */
-    pl3_find2_browser_destroy,                                   /* Destroy */
-	&find2_browser_gbp,		        /* Browser */
-	pl3_find2_browser_status_changed,			                /* status changed */
-	pl3_find2_browser_connection_changed, 	/* connection changed */
-	NULL,		                        /* Preferences */
-	NULL,			                /* MetaData */
-	NULL,                                   /* get_enabled */
-	NULL                                    /* set_enabled */
+	.name = 						"Database Search Browser",
+	.version = 						{1,1,1},
+	.plugin_type = 					GMPC_PLUGIN_PL_BROWSER,
+    .destroy = 						pl3_find2_browser_destroy,
+	.browser = 						&find2_browser_gbp,
+	.mpd_status_changed = 			pl3_find2_browser_status_changed,
+	.mpd_connection_changed = 		pl3_find2_browser_connection_changed,
+	.save_yourself = 				pl3_find2_save_myself,
 };
 
 
 typedef struct {
     GtkWidget *hbox;
     GtkWidget *combo;
+    GtkWidget  *lcombo;
     GtkWidget *entry;
 }crit_struct;
 
@@ -101,16 +98,16 @@ static GtkTreeRowReference *pl3_find2_ref = NULL;
 extern GladeXML *pl3_xml;
 
 /* internal */
-GtkWidget 	*pl3_find2_tree 	= NULL;
-GmpcMpdDataModel *pl3_find2_store2 = NULL;
-GtkWidget 	*pl3_find2_vbox 	= NULL;
-GtkWidget	*pl3_find2_findbut     = NULL;
-GtkWidget       *pl3_find2_critaddbut   = NULL;
-GtkListStore	*pl3_find2_combo_store 	= NULL;
-GtkWidget	*pl3_find2_pb = NULL;
-
-GList *criterias = NULL;
-GtkWidget *pl3_find2_crit_vbox = NULL;
+GtkWidget 			*pl3_find2_tree 		= NULL;
+GmpcMpdDataModel 	*pl3_find2_store2 		= NULL;
+GtkWidget 			*pl3_find2_vbox 		= NULL;
+GtkWidget			*pl3_find2_findbut		= NULL;
+GtkWidget       	*pl3_find2_critaddbut   = NULL;
+GtkListStore		*pl3_find2_combo_store 	= NULL;
+GtkWidget			*pl3_find2_pb 			= NULL;
+static GtkListStore 		*pl3_find2_autocomplete = NULL;
+GList 				*criterias 				= NULL;
+GtkWidget 			*pl3_find2_crit_vbox 	= NULL;
 
 static void pl3_find2_fill_combo()
 {
@@ -155,9 +152,18 @@ static void pl3_find2_browser_add_crit()
     crit_struct *cs = g_malloc0(sizeof(*cs));
     GtkWidget *removebut = NULL;
     GtkCellRenderer *renderer = NULL;
+	GtkEntryCompletion *ent_comp = NULL;
 
     cs->hbox = gtk_hbox_new(FALSE, 6);
+
+    cs->lcombo = gtk_combo_box_new_text();
+    gtk_combo_box_append_text(GTK_COMBO_BOX(cs->lcombo), _("And"));
+    gtk_combo_box_append_text(GTK_COMBO_BOX(cs->lcombo), _("Or"));
+    gtk_combo_box_set_active(GTK_COMBO_BOX(cs->lcombo), 0);
+    gtk_box_pack_start(GTK_BOX(cs->hbox), cs->lcombo, FALSE, TRUE, 0);
+
     cs->combo= gtk_combo_box_new();
+
     renderer = gtk_cell_renderer_text_new();
     gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(cs->combo), renderer, TRUE);
     gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(cs->combo), renderer, "text", 1, NULL);
@@ -166,7 +172,22 @@ static void pl3_find2_browser_add_crit()
     gtk_box_pack_start(GTK_BOX(cs->hbox), cs->combo, FALSE, TRUE, 0);
     g_signal_connect(G_OBJECT(cs->combo), "changed", G_CALLBACK(pl3_find2_combo_box_changed), NULL);
 
+
+
+
+
+
+
+
     cs->entry = gtk_entry_new();
+	ent_comp = gtk_entry_completion_new();
+	gtk_entry_completion_set_text_column(ent_comp, 0);
+	gtk_entry_completion_set_inline_completion(ent_comp, TRUE);
+	gtk_entry_completion_set_model(GTK_ENTRY_COMPLETION(ent_comp), GTK_TREE_MODEL(pl3_find2_autocomplete));
+	gtk_entry_completion_set_popup_completion(GTK_ENTRY_COMPLETION(ent_comp), TRUE);
+	gtk_entry_set_completion(GTK_ENTRY(cs->entry), ent_comp);
+
+
     g_signal_connect(G_OBJECT(cs->entry), "activate",G_CALLBACK(pl3_find2_browser_search), NULL);
     gtk_box_pack_start(GTK_BOX(cs->hbox), cs->entry, TRUE, TRUE, 0);
 
@@ -196,6 +217,8 @@ static void pl3_find2_browser_init()
 {
     GtkWidget  *pl3_find2_sw = NULL;
     GtkWidget *hbox = NULL;
+	/* autocomplete later on */
+	pl3_find2_autocomplete = gtk_list_store_new(1, G_TYPE_STRING);
 
 	pl3_find2_store2 = gmpc_mpddata_model_new();
 
@@ -210,8 +233,8 @@ static void pl3_find2_browser_init()
 
     /* setup signals */
     g_signal_connect(G_OBJECT(pl3_find2_tree), "row-activated",G_CALLBACK(pl3_find2_browser_row_activated), NULL); 
-    g_signal_connect(G_OBJECT(pl3_find2_tree), "button-press-event", G_CALLBACK(pl3_find2_browser_button_release_event), NULL);
-//    g_signal_connect(G_OBJECT(pl3_find2_tree), "button-release-event", G_CALLBACK(pl3_find2_browser_button_release_event), NULL);
+    g_signal_connect(G_OBJECT(pl3_find2_tree), "button-press-event", G_CALLBACK(pl3_find2_browser_button_press_event), NULL);
+    g_signal_connect(G_OBJECT(pl3_find2_tree), "button-release-event", G_CALLBACK(pl3_find2_browser_button_release_event), NULL);
     g_signal_connect(G_OBJECT(pl3_find2_tree), "key-press-event", G_CALLBACK(pl3_find2_browser_playlist_key_press), NULL);
 
     /* set up the scrolled window */
@@ -274,7 +297,6 @@ static void pl3_find2_browser_selected(GtkWidget *container)
     {
         pl3_find2_browser_init();
     }
-
     gtk_container_add(GTK_CONTAINER(container),pl3_find2_vbox);
     gtk_widget_grab_focus(pl3_find2_tree);
     gtk_widget_show(pl3_find2_vbox);
@@ -295,7 +317,9 @@ static void pl3_find2_browser_add(GtkWidget *cat_tree)
 {
     GtkTreePath *path = NULL;
     GtkTreeIter iter;
-    gtk_tree_store_append(pl3_tree, &iter, NULL);
+	gint pos = cfg_get_single_value_as_int_with_default(config, "find2-browser","position",4);
+	playlist3_insert_browser(&iter, pos);
+   // gtk_tree_store_append(pl3_tree, &iter, NULL);
     gtk_tree_store_set(pl3_tree, &iter, 
             PL3_CAT_TYPE, find2_browser_plug.id,
             PL3_CAT_TITLE, _("Database Search"),
@@ -322,14 +346,15 @@ static unsigned long pl3_find2_browser_view_browser()
     GList *node = NULL;
     int found = 0;
 
-    MpdData *data = NULL;
+    MpdData *data_t = NULL, *data =NULL;
 
     gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_find2_tree), NULL);
     if(criterias == NULL)
     {
         return 0;
-    }
-    for(node= g_list_first(criterias); node; node = g_list_next(node))
+    }   
+    node= g_list_first(criterias);
+    for(; node; node = g_list_next(node))
     {
         GtkTreeIter cc_iter;
         int num_field;
@@ -344,13 +369,44 @@ static unsigned long pl3_find2_browser_view_browser()
             }
             gtk_tree_model_get(GTK_TREE_MODEL(pl3_find2_combo_store),&cc_iter , 0, &num_field, -1);
             mpd_database_search_add_constraint(connection, num_field, (char *)name);
+            /* hack to correctly update the autocompletion. damn I must write something that does this more efficient */
+            {
+                GtkTreeIter iter;
+                gboolean found2 = FALSE;
+                for(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(pl3_find2_autocomplete), &iter);
+                        gtk_list_store_iter_is_valid(pl3_find2_autocomplete, &iter) && !found2;
+                        gtk_tree_model_iter_next(GTK_TREE_MODEL(pl3_find2_autocomplete), &iter))
+                {
+                    gchar *entry;
+                    gtk_tree_model_get(GTK_TREE_MODEL(pl3_find2_autocomplete), &iter, 0,&entry,-1);
+                    if(strcmp(entry, name) == 0)
+                    {
+                        found2 = TRUE;
+                    }
+                    g_free(entry);
+                }
+                if(!found2) {
+                    gtk_list_store_insert_with_values(pl3_find2_autocomplete, &iter,-1, 0,name,-1);
+                }					
+            }
+        }
+        if(node->next)
+        {
+            crit_struct *cs2 = node->next->data;
+            if(gtk_combo_box_get_active(GTK_COMBO_BOX(cs2->lcombo))==1)
+            {
+                data = mpd_database_search_commit(connection);
+                data_t = mpd_data_concatenate(data_t, data);
+                data = NULL;
+                found = FALSE;
+            }
         }
     }
-    if(!found)
-        return 0;
-
-    data = mpd_database_search_commit(connection);
-	gmpc_mpddata_model_set_mpd_data(pl3_find2_store2, data);
+    if(found)
+        data = mpd_database_search_commit(connection);
+    data_t = mpd_data_concatenate(data_t, data);
+    data_t = misc_mpddata_remove_duplicate_songs(data_t);
+    gmpc_mpddata_model_set_mpd_data(pl3_find2_store2, data_t);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_find2_tree), GTK_TREE_MODEL(pl3_find2_store2));
     return time;
 }
@@ -382,9 +438,9 @@ static void pl3_find2_browser_show_info()
     {
         GList *list = NULL;
         list = gtk_tree_selection_get_selected_rows (selection, &model);
-        /* iterate over every row */
+        
         list = g_list_last (list);
-        do
+  //      do
         {
             GtkTreeIter iter;
             mpd_Song *song =NULL;
@@ -393,12 +449,15 @@ static void pl3_find2_browser_show_info()
             gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) list->data);
             gtk_tree_model_get(model,&iter,MPDDATA_MODEL_COL_PATH, &path,-1);
             song = mpd_database_get_fileinfo(connection, path);
-            if(song)
-                call_id3_window_song(song); 
+			if(song) {
+				info2_activate();
+				info2_fill_song_view(song);	
+				mpd_freeSong(song);
+			}
             q_free(path);
         }
-        while ((list = g_list_previous (list)) && mpd_check_connected(connection));
-        /* free list */
+//        while ((list = g_list_previous (list)) && mpd_check_connected(connection));
+
         g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
         g_list_free (list);
     }
@@ -421,9 +480,11 @@ static void pl3_find2_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp)
 static void pl3_find2_browser_replace_selected()
 {
     mpd_playlist_clear(connection);
-    pl3_find2_browser_add_selected();
-    mpd_player_play(connection);	
-
+    if(mpd_check_connected(connection))
+    {
+        pl3_find2_browser_add_selected();
+        mpd_player_play(connection);	
+    }
 }
 
 static int pl3_find2_browser_playlist_key_press(GtkWidget *tree, GdkEventKey *event)
@@ -493,6 +554,24 @@ static void pl3_find2_browser_edit_columns(void)
   gmpc_mpddata_treeview_edit_columns(GMPC_MPDDATA_TREEVIEW(pl3_find2_tree));
 
 }
+static gboolean pl3_find2_browser_button_press_event(GtkWidget *giv, GdkEventButton *event, gpointer data)
+{
+	GtkTreePath *path = NULL;
+	if(event->button == 3 &&gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(giv), event->x, event->y,&path,NULL,NULL,NULL))
+	{	
+		GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(giv));
+		if(gtk_tree_selection_path_is_selected(sel, path))
+		{
+			gtk_tree_path_free(path);
+			return TRUE;
+		}
+	}
+	if(path) {
+		gtk_tree_path_free(path);
+	}
+	return FALSE;                                                                                                     
+}
+
 
 static gboolean pl3_find2_browser_button_release_event(GtkWidget *but, GdkEventButton *event)
 {
@@ -554,7 +633,7 @@ static gboolean pl3_find2_browser_button_release_event(GtkWidget *but, GdkEventB
       gtk_widget_show(item);
 
       gtk_widget_show_all(menu);
-      gtk_menu_popup(GTK_MENU(menu), NULL, NULL,NULL, NULL, event->button, event->time);
+      gtk_menu_popup(GTK_MENU(menu), NULL, NULL,NULL, NULL, 0, event->time);
       return TRUE;
     }
     return FALSE;
@@ -586,6 +665,7 @@ static int pl3_find2_browser_add_go_menu(GtkWidget *menu)
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
             gtk_image_new_from_icon_name("gtk-find", GTK_ICON_SIZE_MENU));
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+	gtk_widget_add_accelerator(GTK_WIDGET(item), "activate", gtk_menu_get_accel_group(GTK_MENU(menu)), GDK_F5, 0, GTK_ACCEL_VISIBLE);
     g_signal_connect(G_OBJECT(item), "activate", 
             G_CALLBACK(pl3_find2_browser_activate), NULL);
 
@@ -602,30 +682,45 @@ static void pl3_find2_browser_connection_changed(MpdObj *mi, int connect, gpoint
 
 static int pl3_find2_browser_key_press_event(GtkWidget *mw, GdkEventKey *event, int type)
 {
-    if (event->keyval == GDK_F5)
+/*    if (event->keyval == GDK_F5)
     {
         pl3_find2_browser_activate();
         return TRUE;
     }                                           	
-
+*/
     return FALSE;
 }
 
 static void pl3_find2_browser_destroy(void)
 {
+  if(pl3_find2_ref)
+  {
+      GtkTreeIter iter;
+      GtkTreePath *path;
+      path = gtk_tree_row_reference_get_path(pl3_find2_ref);
+      if(path)
+      {
+          if(gtk_tree_model_get_iter(GTK_TREE_MODEL(gtk_tree_row_reference_get_model(pl3_find2_ref)), &iter,path))
+          {
+              gtk_tree_store_remove(GTK_TREE_STORE(gtk_tree_row_reference_get_model(pl3_find2_ref)), &iter);
+          }
+          gtk_tree_path_free(path);
+      }
+      gtk_tree_row_reference_free(pl3_find2_ref);
+      pl3_find2_ref = NULL;
+  }
   if(pl3_find2_vbox)
   {
     gtk_widget_destroy(pl3_find2_vbox);
+    pl3_find2_vbox = NULL;
   }
   if(pl3_find2_store2)
   {
     g_object_unref(pl3_find2_store2);
-  }
-  if(pl3_find2_ref)
-  {
-    gtk_tree_row_reference_free(pl3_find2_ref);
+    pl3_find2_store2 = NULL;
   }
 }
+
 static void pl3_find2_browser_status_changed(MpdObj *mi,ChangedStatusType what, void *data)
 {
 	if(what&MPD_CST_DATABASE)
@@ -633,3 +728,18 @@ static void pl3_find2_browser_status_changed(MpdObj *mi,ChangedStatusType what,
 		pl3_find2_browser_search(); 
 	}
 }	
+static void pl3_find2_save_myself(void)
+{
+	if(pl3_find2_ref)
+	{
+		GtkTreePath *path = gtk_tree_row_reference_get_path(pl3_find2_ref);
+		if(path)
+		{
+			gint *indices = gtk_tree_path_get_indices(path);
+			debug_printf(DEBUG_INFO,"Saving myself to position: %i\n", indices[0]);
+			cfg_set_single_value_as_int(config, "find2-browser","position",indices[0]);
+			gtk_tree_path_free(path);
+		}
+	}
+}
+
