@@ -30,7 +30,6 @@
 #include "playlist3.h"
 #include "playlist3-find3-browser.h"
 #include "config1.h"
-#include "id3info.h"
 
 #include "eggcolumnchooserdialog.h"
 
@@ -49,6 +48,7 @@ static int pl3_find3_browser_add_go_menu(GtkWidget *);
 static void pl3_find3_browser_search(void);
 static void pl3_find3_browser_row_activated(GtkTreeView *, GtkTreePath *);
 static int pl3_find3_browser_playlist_key_press(GtkWidget *, GdkEventKey *);
+static gboolean pl3_find3_browser_button_press_event(GtkWidget *but, GdkEventButton *event,gpointer data);
 static gboolean pl3_find3_browser_button_release_event(GtkWidget *but, GdkEventButton *event);
 static void pl3_find3_browser_connection_changed(MpdObj *mi, int connect, gpointer data);
 static int pl3_find3_browser_key_press_event(GtkWidget *mw, GdkEventKey *event, int type);
@@ -57,6 +57,8 @@ extern GladeXML *pl3_xml;
 
 static void pl3_find3_combo_box_changed(GtkComboBox *cb, gpointer data);
 
+static void pl3_find3_save_myself(void);
+
 int pl3_find3_last_entry = MPD_TAG_ITEM_ANY;
 /**
  * Plugin structure
@@ -74,26 +76,21 @@ gmpcPlBrowserPlugin find3_browser_gbp = {
 };
 
 gmpcPlugin find3_browser_plug = {
-    "Playlist Browser",
-    {1,1,1},
-    GMPC_PLUGIN_PL_BROWSER,
-    0,
-    NULL,		                	/* path*/
-    NULL,			                /* init */
-    pl3_find3_browser_destroy,      /* Destroy */
-    &find3_browser_gbp,	        	/* Browser */
-    pl3_find3_browser_status_changed,		/* status changed */
-    pl3_find3_browser_connection_changed, 	/* connection changed */
-    NULL,		                    /* Preferences */
-    NULL,			                /* MetaData */
-    NULL,                           /* get_enable */
-    NULL                            /* set_enable */
+   	.name = 						 "Playlist Browser",
+	.version = 					    {1,1,1},
+	.plugin_type = 				    GMPC_PLUGIN_PL_BROWSER,
+    .destroy = 						pl3_find3_browser_destroy,  
+    .browser = 						&find3_browser_gbp,	
+    .mpd_status_changed = 			pl3_find3_browser_status_changed,
+    .mpd_connection_changed = 		pl3_find3_browser_connection_changed,
+	.save_yourself = 				pl3_find3_save_myself,
 };
 
 
 typedef struct {
     GtkWidget *hbox;
     GtkWidget *combo;
+    GtkWidget *lcombo;
     GtkWidget *entry;
     regex_t preq;
     int tag_type; 
@@ -103,7 +100,7 @@ typedef struct {
 
 /* Playlist window row reference */
 static GtkTreeRowReference *pl3_find3_ref = NULL;
-
+static GtkListStore 		*pl3_find3_autocomplete = NULL;
 
 extern GladeXML *pl3_xml;
 
@@ -162,8 +159,20 @@ static void pl3_find3_browser_add_crit()
     crit3_struct *cs = g_malloc0(sizeof(*cs));
     GtkWidget *removebut = NULL;
     GtkCellRenderer *renderer = NULL;
-
+	GtkEntryCompletion *ent_comp;
+    cs->tag_type = -1;
     cs->hbox = gtk_hbox_new(FALSE, 6);
+    cs->lcombo = gtk_combo_box_new_text();
+    gtk_combo_box_append_text(GTK_COMBO_BOX(cs->lcombo), _("And"));
+    gtk_combo_box_append_text(GTK_COMBO_BOX(cs->lcombo), _("Or"));
+    gtk_combo_box_set_active(GTK_COMBO_BOX(cs->lcombo), 0);
+	if(mpd_server_check_command_allowed(connection, "playlistsearch")== MPD_SERVER_COMMAND_ALLOWED && 
+			mpd_server_check_command_allowed(connection, "playlistfind")== MPD_SERVER_COMMAND_ALLOWED/*&&
+            criterias3*/)
+    {
+        gtk_box_pack_start(GTK_BOX(cs->hbox), cs->lcombo, FALSE, TRUE, 0);
+    }
+
     cs->combo= gtk_combo_box_new();
     renderer = gtk_cell_renderer_text_new();
     gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(cs->combo), renderer, TRUE);
@@ -174,6 +183,19 @@ static void pl3_find3_browser_add_crit()
     g_signal_connect(G_OBJECT(cs->combo), "changed", G_CALLBACK(pl3_find3_combo_box_changed), NULL);
 
     cs->entry = gtk_entry_new();
+	ent_comp = gtk_entry_completion_new();
+	gtk_entry_completion_set_text_column(ent_comp, 0);
+	gtk_entry_completion_set_inline_completion(ent_comp, TRUE);
+	gtk_entry_completion_set_model(GTK_ENTRY_COMPLETION(ent_comp), GTK_TREE_MODEL(pl3_find3_autocomplete));
+	gtk_entry_completion_set_popup_completion(GTK_ENTRY_COMPLETION(ent_comp), TRUE);
+	gtk_entry_set_completion(GTK_ENTRY(cs->entry), ent_comp);
+
+
+
+
+
+
+
     g_signal_connect(G_OBJECT(cs->entry), "activate",G_CALLBACK(pl3_find3_browser_search), NULL);
     gtk_box_pack_start(GTK_BOX(cs->hbox), cs->entry, TRUE, TRUE, 0);
 
@@ -204,8 +226,10 @@ static void pl3_find3_browser_init()
     GtkWidget  *pl3_find3_sw = NULL;
     GtkWidget *hbox = NULL;
 
-	pl3_find3_store2 = gmpc_mpddata_model_new();
+	pl3_find3_autocomplete = gtk_list_store_new(1, G_TYPE_STRING);
 
+	pl3_find3_store2 = gmpc_mpddata_model_new();
+    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(pl3_find3_store2), GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
 
     pl3_find3_combo_store = gtk_list_store_new(2,G_TYPE_INT, G_TYPE_STRING);
     /** Fill the view */
@@ -216,8 +240,8 @@ static void pl3_find3_browser_init()
 
     /* setup signals */
     g_signal_connect(G_OBJECT(pl3_find3_tree), "row-activated",G_CALLBACK(pl3_find3_browser_row_activated), NULL); 
-    g_signal_connect(G_OBJECT(pl3_find3_tree), "button-press-event", G_CALLBACK(pl3_find3_browser_button_release_event), NULL);
-//    g_signal_connect(G_OBJECT(pl3_find3_tree), "button-release-event", G_CALLBACK(pl3_find3_browser_button_release_event), NULL);
+    g_signal_connect(G_OBJECT(pl3_find3_tree), "button-press-event", G_CALLBACK(pl3_find3_browser_button_press_event), NULL);
+    g_signal_connect(G_OBJECT(pl3_find3_tree), "button-release-event", G_CALLBACK(pl3_find3_browser_button_release_event), NULL);
     g_signal_connect(G_OBJECT(pl3_find3_tree), "key-press-event", G_CALLBACK(pl3_find3_browser_playlist_key_press), NULL);
 
     /* set up the scrolled window */
@@ -296,7 +320,10 @@ static void pl3_find3_browser_selected(GtkWidget *container)
 }
 static void pl3_find3_browser_unselected(GtkWidget *container)
 {
-    gtk_container_remove(GTK_CONTAINER(container), pl3_find3_vbox);
+	if(pl3_find3_vbox)
+	{
+	    gtk_container_remove(GTK_CONTAINER(container), pl3_find3_vbox);
+	}
 }
 
 /*****************************************************************
@@ -307,7 +334,9 @@ static void pl3_find3_browser_add(GtkWidget *cat_tree)
 {
     GtkTreePath *path = NULL;
     GtkTreeIter iter;
-    gtk_tree_store_append(pl3_tree, &iter, NULL);
+	gint pos = cfg_get_single_value_as_int_with_default(config, "find3-browser","position",1);
+	playlist3_insert_browser(&iter, pos);
+//    gtk_tree_store_append(pl3_tree, &iter, NULL);
     gtk_tree_store_set(pl3_tree, &iter, 
             PL3_CAT_TYPE, find3_browser_plug.id,
             PL3_CAT_TITLE, _("Playlist Search"),
@@ -332,175 +361,244 @@ static unsigned long pl3_find3_browser_view_browser_old_style()
 {
 	MpdData *data = NULL;
     int time=0;
-    GList *node = NULL;
+    GList *node2,*node = NULL;
     int found = 0;
 
     if(criterias3 == NULL)
         return 0;
     /** check rules, see if there is a usefull one, if so compile a regex */
-    for(node= g_list_first(criterias3); node; node = g_list_next(node))
+    gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_find3_tree), NULL);
+    node= g_list_first(criterias3);
+    while(node)
     {
-        GtkTreeIter cc_iter;
-        crit3_struct *cs = node->data;
-        const gchar *name = gtk_entry_get_text(GTK_ENTRY(cs->entry));
-        cs->tag_type = -1;
-        if(gtk_combo_box_get_active_iter(GTK_COMBO_BOX(cs->combo), &cc_iter) && name && name[0] != '\0')
+        found = FALSE;
+        for(; node; node = g_list_next(node))
         {
-            if(!regcomp(&(cs->preq),name, REG_EXTENDED|REG_ICASE))
+            GtkTreeIter cc_iter;
+            crit3_struct *cs = node->data;
+            const gchar *name = gtk_entry_get_text(GTK_ENTRY(cs->entry));
+            cs->tag_type = -1;
+            if(gtk_combo_box_get_active_iter(GTK_COMBO_BOX(cs->combo), &cc_iter) && name && name[0] != '\0')
             {
-                int num_field;
-                found = TRUE;
-                gtk_tree_model_get(GTK_TREE_MODEL(pl3_find3_combo_store),&cc_iter , 0, &num_field, -1);
-                cs->tag_type = num_field;
-            } else {
-                regfree(&(cs->preq));
+                if(!regcomp(&(cs->preq),name, REG_EXTENDED|REG_ICASE))
+                {
+                    int num_field;
+                    found = TRUE;
+                    gtk_tree_model_get(GTK_TREE_MODEL(pl3_find3_combo_store),&cc_iter , 0, &num_field, -1);
+                    cs->tag_type = num_field;
+                } else {
+                    regfree(&(cs->preq));
+                    cs->tag_type = -1;
+                }
+            }
+            if(node->next)
+            {
+                crit3_struct *cs2 = node->next->data;
+                if(gtk_combo_box_get_active(GTK_COMBO_BOX(cs2->lcombo)) == 1)
+                {
+                    node = node->next;
+                    break;
+                }
             }
         }
-    }
-    if(!found)
-        return 0;
-    /* get markup */
-    gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_find3_tree), NULL);
-    /** Fill now */
-    {
-        GtkTreeIter iter;
-        if(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playlist), &iter))
-        {
-            GList *node = NULL;
-            int songs = 0, total_songs = mpd_playlist_get_playlist_length(connection);
-            int step = total_songs/50;
-            if(playlist_list_get_loaded(PLAYLIST_LIST(playlist)) < 1)
+
+        /** Fill now */
+        if(found){
+            GtkTreeIter iter;
+            if(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playlist), &iter))
             {
-                gtk_widget_show(pl3_find3_pb);
-                gtk_widget_set_sensitive(glade_xml_get_widget(pl3_xml, "pl3_win"),FALSE);
-            }
-            else{
-                total_songs = 0;
-            }
-            do {
-                int loop = TRUE;
-                mpd_Song *song = NULL;
-                gtk_tree_model_get(GTK_TREE_MODEL(playlist), &iter, PLAYLIST_LIST_COL_MPDSONG, &song, -1);
-
-
-                songs++;
-                if(step && total_songs && (songs % step) == 0)
+                GList *node3 = NULL;
+                int songs = 0, total_songs = mpd_playlist_get_playlist_length(connection);
+                int step = total_songs/50;
+                if(playlist_list_get_loaded(PLAYLIST_LIST(playlist)) < 1)
                 {
-                    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pl3_find3_pb), songs/(gdouble)total_songs);
-                    while(gtk_events_pending()) gtk_main_iteration();
+                    gtk_widget_show(pl3_find3_pb);
+                    gtk_widget_set_sensitive(glade_xml_get_widget(pl3_xml, "pl3_win"),FALSE);
                 }
+                else{
+                    total_songs = 0;
+                }
+                do {
+                    int loop = TRUE;
+                    mpd_Song *song = NULL;
+                    gtk_tree_model_get(GTK_TREE_MODEL(playlist), &iter, PLAYLIST_LIST_COL_MPDSONG, &song, -1);
 
-                if((node = g_list_first(criterias3)))
-                {
-                    loop = FALSE;
-                    for(;!loop && node; node = g_list_next(node))
+
+                    songs++;
+                    if(step && total_songs && (songs % step) == 0)
                     {
-                        loop = TRUE;
+                        gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pl3_find3_pb), songs/(gdouble)total_songs);
+                        while(gtk_events_pending()) gtk_main_iteration();
+                    }
 
-                        crit3_struct *cs = node->data;
-                        if(cs && cs->tag_type >= 0)
+                    if((node3 = g_list_first(criterias3)))
+                    {
+                        loop = FALSE;
+                        for(;!loop && node3; node3 = g_list_next(node3))
                         {
-                            if(song->artist && (cs->tag_type == MPD_TAG_ITEM_ARTIST || cs->tag_type == MPD_TAG_ITEM_ANY)){
-                                if(regexec(&(cs->preq),song->artist,0,NULL,0) == 0)loop = FALSE; 
-                            } 
-                            if(song->album && (cs->tag_type == MPD_TAG_ITEM_ALBUM || cs->tag_type == MPD_TAG_ITEM_ANY)){
-                                if(regexec(&(cs->preq),song->album,0,NULL,0) == 0)loop = FALSE; 
-                            }
-                            if(song->title && (cs->tag_type == MPD_TAG_ITEM_TITLE || cs->tag_type == MPD_TAG_ITEM_ANY)){
-                                if(regexec(&(cs->preq),song->title,0,NULL,0) == 0)loop = FALSE; 
-                            }
-                            if(song->track && (cs->tag_type == MPD_TAG_ITEM_TRACK || cs->tag_type == MPD_TAG_ITEM_ANY)){
-                                if(regexec(&(cs->preq),song->track,0,NULL,0) == 0)loop = FALSE; 
-                            }
-                            if(song->name && (cs->tag_type == MPD_TAG_ITEM_NAME || cs->tag_type == MPD_TAG_ITEM_ANY)){
-                                if(regexec(&(cs->preq),song->name,0,NULL,0) == 0)loop = FALSE; 
-                            }
-                            if(song->genre&& (cs->tag_type == MPD_TAG_ITEM_GENRE|| cs->tag_type == MPD_TAG_ITEM_ANY)){
-                                if(regexec(&(cs->preq),song->genre,0,NULL,0) == 0)loop = FALSE; 
-                            }
-                            if(song->date&& (cs->tag_type == MPD_TAG_ITEM_DATE|| cs->tag_type == MPD_TAG_ITEM_ANY)){
-                                if(regexec(&(cs->preq),song->date,0,NULL,0) == 0)loop = FALSE; 
-                            }
-                            if(song->composer&& (cs->tag_type == MPD_TAG_ITEM_COMPOSER|| cs->tag_type == MPD_TAG_ITEM_ANY)){
-                                if(regexec(&(cs->preq),song->composer,0,NULL,0) == 0)loop = FALSE; 
-                            }
-							if(song->performer&& (cs->tag_type == MPD_TAG_ITEM_PERFORMER||cs->tag_type == MPD_TAG_ITEM_ANY)){
-								if(regexec(&(cs->preq),song->performer,0,NULL,0) == 0)loop = FALSE; 
-							}
-                            if(song->comment&& (cs->tag_type == MPD_TAG_ITEM_COMMENT||cs->tag_type == MPD_TAG_ITEM_ANY)){
-                                if(regexec(&(cs->preq),song->comment,0,NULL,0) == 0)loop = FALSE; 
-                            }                                                                        
-                            if(song->disc&& (cs->tag_type == MPD_TAG_ITEM_DISC||cs->tag_type == MPD_TAG_ITEM_ANY)){
-                                if(regexec(&(cs->preq),song->disc,0,NULL,0) == 0)loop = FALSE; 
-                            }
-                            if(song->file&& (cs->tag_type == MPD_TAG_ITEM_FILENAME||cs->tag_type == MPD_TAG_ITEM_ANY)){
-                                if(regexec(&(cs->preq),song->file,0,NULL,0) == 0)loop = FALSE; 
+                            loop = TRUE;
+
+                            crit3_struct *cs = node3->data;
+                            if(cs && cs->tag_type >= 0)
+                            {
+                                if(song->artist && (cs->tag_type == MPD_TAG_ITEM_ARTIST || cs->tag_type == MPD_TAG_ITEM_ANY)){
+                                    if(regexec(&(cs->preq),song->artist,0,NULL,0) == 0)loop = FALSE; 
+                                } 
+                                if(song->album && (cs->tag_type == MPD_TAG_ITEM_ALBUM || cs->tag_type == MPD_TAG_ITEM_ANY)){
+                                    if(regexec(&(cs->preq),song->album,0,NULL,0) == 0)loop = FALSE; 
+                                }
+                                if(song->title && (cs->tag_type == MPD_TAG_ITEM_TITLE || cs->tag_type == MPD_TAG_ITEM_ANY)){
+                                    if(regexec(&(cs->preq),song->title,0,NULL,0) == 0)loop = FALSE; 
+                                }
+                                if(song->track && (cs->tag_type == MPD_TAG_ITEM_TRACK || cs->tag_type == MPD_TAG_ITEM_ANY)){
+                                    if(regexec(&(cs->preq),song->track,0,NULL,0) == 0)loop = FALSE; 
+                                }
+                                if(song->name && (cs->tag_type == MPD_TAG_ITEM_NAME || cs->tag_type == MPD_TAG_ITEM_ANY)){
+                                    if(regexec(&(cs->preq),song->name,0,NULL,0) == 0)loop = FALSE; 
+                                }
+                                if(song->genre&& (cs->tag_type == MPD_TAG_ITEM_GENRE|| cs->tag_type == MPD_TAG_ITEM_ANY)){
+                                    if(regexec(&(cs->preq),song->genre,0,NULL,0) == 0)loop = FALSE; 
+                                }
+                                if(song->date&& (cs->tag_type == MPD_TAG_ITEM_DATE|| cs->tag_type == MPD_TAG_ITEM_ANY)){
+                                    if(regexec(&(cs->preq),song->date,0,NULL,0) == 0)loop = FALSE; 
+                                }
+                                if(song->composer&& (cs->tag_type == MPD_TAG_ITEM_COMPOSER|| cs->tag_type == MPD_TAG_ITEM_ANY)){
+                                    if(regexec(&(cs->preq),song->composer,0,NULL,0) == 0)loop = FALSE; 
+                                }
+                                if(song->performer&& (cs->tag_type == MPD_TAG_ITEM_PERFORMER||cs->tag_type == MPD_TAG_ITEM_ANY)){
+                                    if(regexec(&(cs->preq),song->performer,0,NULL,0) == 0)loop = FALSE; 
+                                }
+                                if(song->comment&& (cs->tag_type == MPD_TAG_ITEM_COMMENT||cs->tag_type == MPD_TAG_ITEM_ANY)){
+                                    if(regexec(&(cs->preq),song->comment,0,NULL,0) == 0)loop = FALSE; 
+                                }                                                                        
+                                if(song->disc&& (cs->tag_type == MPD_TAG_ITEM_DISC||cs->tag_type == MPD_TAG_ITEM_ANY)){
+                                    if(regexec(&(cs->preq),song->disc,0,NULL,0) == 0)loop = FALSE; 
+                                }
+                                if(song->file&& (cs->tag_type == MPD_TAG_ITEM_FILENAME||cs->tag_type == MPD_TAG_ITEM_ANY)){
+                                    if(regexec(&(cs->preq),song->file,0,NULL,0) == 0)loop = FALSE; 
+                                }
                             }
+                            else
+                                loop = FALSE;
                         }
                     }
+                    if(!loop)
+                    {
+                        mpd_Song *song = NULL;
+                        gtk_tree_model_get(GTK_TREE_MODEL(playlist), &iter, PLAYLIST_LIST_COL_MPDSONG, &song, -1);
+                        data = mpd_new_data_struct_append(data);
+                        data->type = MPD_DATA_TYPE_SONG;
+                        data->song = mpd_songDup(song);  
+                    }
+                } while(gtk_tree_model_iter_next(GTK_TREE_MODEL(playlist), &iter));
+
+                /* remove the progress bar */
+                if(total_songs) {
+                    gtk_widget_hide(pl3_find3_pb);   		   
+                    gtk_widget_set_sensitive(glade_xml_get_widget(pl3_xml, "pl3_win"),TRUE);
                 }
-                if(!loop)
-		{
-			mpd_Song *song = NULL;
-			gtk_tree_model_get(GTK_TREE_MODEL(playlist), &iter, PLAYLIST_LIST_COL_MPDSONG, &song, -1);
-			data = mpd_new_data_struct_append(data);
-			data->type = MPD_DATA_TYPE_SONG;
-			data->song = mpd_songDup(song);  
-		}
-            } while(gtk_tree_model_iter_next(GTK_TREE_MODEL(playlist), &iter));
-			gmpc_mpddata_model_set_mpd_data(pl3_find3_store2, data);
-            /* remove the progress bar */
-            if(total_songs) {
-                gtk_widget_hide(pl3_find3_pb);   		   
-                gtk_widget_set_sensitive(glade_xml_get_widget(pl3_xml, "pl3_win"),TRUE);
             }
         }
+        /* cleanup the regex compiled */
+        for(node2= g_list_first(criterias3); node2 ; node2 = g_list_next(node2)) {
+            crit3_struct *cs = node2->data;
+            if(cs->tag_type >= 0)
+                regfree(&(cs->preq));
+            cs->tag_type = -1;
+        }
     }
-    /* cleanup the regex compiled */
-    for(node= g_list_first(criterias3); node; node = g_list_next(node)) {
-        crit3_struct *cs = node->data;
-        if(cs->tag_type >= 0)
-            regfree(&(cs->preq));
-        cs->tag_type = -1;
-    }
+    data = misc_mpddata_remove_duplicate_songs(data);
+    gmpc_mpddata_model_set_mpd_data(pl3_find3_store2, data);
     /* set model again */
     gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_find3_tree), GTK_TREE_MODEL(pl3_find3_store2));
     return time;
 }
+
+static gint __position_sort(gpointer aa, gpointer bb, gpointer data)
+{
+    MpdData_real *a = *(MpdData_real **)aa;
+    MpdData_real *b = *(MpdData_real **)bb;
+    return a->song->pos - b->song->pos;         
+}
+
+
+
+
 static unsigned long pl3_find3_browser_view_browser()
 {
 	if(mpd_server_check_command_allowed(connection, "playlistsearch")== MPD_SERVER_COMMAND_ALLOWED && 
 			mpd_server_check_command_allowed(connection, "playlistfind")== MPD_SERVER_COMMAND_ALLOWED)
-	{
-		int time=0;
-		GList *node = NULL;
-		int found = 0;
-		gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_find3_tree), NULL);
-		if(criterias3 == NULL)
-			return 0;
-		/** check rules, see if there is a usefull one, if so compile a regex */
-		for(node= g_list_first(criterias3); node; node = g_list_next(node))
-		{
-			GtkTreeIter cc_iter;
-			crit3_struct *cs = node->data;
-			const gchar *name = gtk_entry_get_text(GTK_ENTRY(cs->entry));
-			cs->tag_type = -1;
-			if(gtk_combo_box_get_active_iter(GTK_COMBO_BOX(cs->combo), &cc_iter) && name && name[0] != '\0')
-			{
-        int num_field;
-        if(!found)
-					mpd_playlist_search_start(connection, FALSE);
-				found = TRUE;
+    {
+        int time=0;
+        GList *node = NULL;
+        int found = 0;
+        MpdData *data = NULL, *data_t= NULL;
+        gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_find3_tree), NULL);
+        if(criterias3 == NULL)
+            return 0;
+        /** check rules, see if there is a usefull one, if so compile a regex */
+        node= g_list_first(criterias3);
+        for(; node; node = g_list_next(node))
+        {
+            GtkTreeIter cc_iter;
+            crit3_struct *cs = node->data;
+            const gchar *name = gtk_entry_get_text(GTK_ENTRY(cs->entry));
+            cs->tag_type = -1;
+            if(gtk_combo_box_get_active_iter(GTK_COMBO_BOX(cs->combo), &cc_iter) && name && name[0] != '\0')
+            {
+                int num_field;
+                if(!found)
+                {
+                    mpd_playlist_search_start(connection, FALSE);
+                    found = TRUE;
+                }
                 gtk_tree_model_get(GTK_TREE_MODEL(pl3_find3_combo_store),&cc_iter , 0, &num_field, -1);
-				mpd_playlist_search_add_constraint(connection, num_field, name);
-			}
-		}
-		if(!found) return 0;	
-		MpdData *data = mpd_playlist_search_commit(connection);
-		time = gmpc_mpddata_model_set_mpd_data(pl3_find3_store2, data);
-		gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_find3_tree), GTK_TREE_MODEL(pl3_find3_store2));
-		return time;
-	}
+                mpd_playlist_search_add_constraint(connection, num_field, name);
+
+                {
+                    GtkTreeIter iter;
+                    gboolean found2 = FALSE;
+                    for(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(pl3_find3_autocomplete), &iter);
+                            gtk_list_store_iter_is_valid(pl3_find3_autocomplete, &iter) && !found2;
+                            gtk_tree_model_iter_next(GTK_TREE_MODEL(pl3_find3_autocomplete), &iter))
+                    {
+                        gchar *entry;
+                        gtk_tree_model_get(GTK_TREE_MODEL(pl3_find3_autocomplete), &iter, 0,&entry,-1);
+                        if(strcmp(entry, name) == 0)
+                        {
+                            found2 = TRUE;
+                        }
+                        g_free(entry);
+                    }
+                    if(!found2) {
+                        gtk_list_store_insert_with_values(pl3_find3_autocomplete, &iter,-1, 0,name,-1);
+                    }					
+                }
+
+            }
+            if(node->next)
+            {
+                crit3_struct *cs2 = node->next->data;
+                if(gtk_combo_box_get_active(GTK_COMBO_BOX(cs2->lcombo))==1)
+                {
+                    data = mpd_database_search_commit(connection);
+                    data_t = mpd_data_concatenate(data_t, data);
+                    data = NULL;
+                    found = FALSE;
+                }
+
+            }
+        }
+        if(found)
+            data = mpd_playlist_search_commit(connection);
+        data_t = mpd_data_concatenate(data_t, data);
+        data_t = misc_mpddata_remove_duplicate_songs(data_t);
+        data_t = misc_sort_mpddata(data_t,(GCompareDataFunc)__position_sort,NULL); 
+        gmpc_mpddata_model_set_mpd_data(pl3_find3_store2, data_t);
+
+        gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_find3_tree), GTK_TREE_MODEL(pl3_find3_store2));
+        return time;
+    }
 	else
 		return pl3_find3_browser_view_browser_old_style();
 }
@@ -534,19 +632,21 @@ static void pl3_find3_browser_show_info()
 	{
 		GList *list = NULL;
 		list = gtk_tree_selection_get_selected_rows (selection, &model);
-		/* iterate over every row */
+
 		list = g_list_last (list);
-		do
+	//	do
 		{
 			GtkTreeIter iter;
+			mpd_Song *song = NULL;
 			GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(pl3_find3_tree));
-			int id;
 			gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) list->data);
-			gtk_tree_model_get(model,&iter,MPDDATA_MODEL_COL_SONG_ID, &id,-1);
-			call_id3_window (id);
+			gtk_tree_model_get(model,&iter,MPDDATA_MODEL_COL_MPDSONG, &song,-1);
+			info2_activate();
+			info2_fill_song_view(song);	
+
 		}
-		while ((list = g_list_previous (list)) && mpd_check_connected(connection));
-		/* free list */
+	//	while ((list = g_list_previous (list)) && mpd_check_connected(connection));
+
 		g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
 		g_list_free (list);
 	}
@@ -588,6 +688,54 @@ static void pl3_find3_browser_edit_columns(void)
 {
   gmpc_mpddata_treeview_edit_columns(GMPC_MPDDATA_TREEVIEW(pl3_find3_tree));
 }
+static gboolean pl3_find3_browser_button_press_event(GtkWidget *but, GdkEventButton *event, gpointer data)
+{
+	GtkTreePath *path = NULL;
+	if(event->button == 3 &&gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(but), event->x, event->y,&path,NULL,NULL,NULL))
+	{	
+		GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(but));
+		if(gtk_tree_selection_path_is_selected(sel, path))
+		{
+			gtk_tree_path_free(path);
+			return TRUE;
+		}
+	}
+	if(path) {
+		gtk_tree_path_free(path);
+	}
+	return FALSE;                                                                                                     
+}
+static void pl3_find3_browser_queue_add()
+{
+	/* grab the selection from the tree */
+	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(pl3_find3_tree));
+	/* check if where connected */
+	/* see if there is a row selected */
+	if (gtk_tree_selection_count_selected_rows (selection) > 0)
+	{
+		GList *list = NULL, *llist = NULL;
+		GtkTreeModel *model = GTK_TREE_MODEL(pl3_find3_store2);
+		/* start a command list */
+		/* grab the selected songs */
+		list = gtk_tree_selection_get_selected_rows (selection, &model);
+		/* grab the last song that is selected */
+		llist = g_list_first (list);
+		/* remove every selected song one by one */
+		do{
+			GtkTreeIter iter;
+			int value;
+			gtk_tree_model_get_iter (model, &iter,(GtkTreePath *) llist->data);
+			gtk_tree_model_get (model, &iter, MPDDATA_MODEL_COL_SONG_ID, &value, -1);
+			mpd_playlist_queue_mpd_queue_add(connection, value);			
+		} while ((llist = g_list_next (llist)));
+		mpd_playlist_queue_commit(connection);
+		/* free list */
+		g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
+		g_list_free (list);
+	}
+	mpd_status_queue_update(connection);
+}
+
 
 static gboolean pl3_find3_browser_button_release_event(GtkWidget *but, GdkEventButton *event)
 {
@@ -597,33 +745,43 @@ static gboolean pl3_find3_browser_button_release_event(GtkWidget *but, GdkEventB
 		GtkWidget *item;
 		GtkWidget *menu = gtk_menu_new();
 
+		/* */
+		if(mpd_server_check_command_allowed(connection, "queueinfo") == MPD_SERVER_COMMAND_ALLOWED)
+		{
+			item = gtk_image_menu_item_new_with_label(_("Add to Queue"));
+			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
+					gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+			g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_find3_browser_queue_add), NULL);		
+			gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+        }
+        if(gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection(GTK_TREE_VIEW(pl3_find3_tree))) == 1)	
+        {
+            mpd_Song *song;
+            GtkTreeIter iter;
+            GtkTreePath *path;
+            GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(pl3_find3_tree));
+            GList *list = gtk_tree_selection_get_selected_rows(gtk_tree_view_get_selection(GTK_TREE_VIEW(pl3_find3_tree)), &model);
+            path = list->data;
+            /* free result */
+            g_list_free(list);
+            if(path && gtk_tree_model_get_iter(model, &iter, path)) {
+                gtk_tree_model_get(model, &iter, MPDDATA_MODEL_COL_MPDSONG, &song, -1);
+                if(song)
+                {
+                    submenu_for_song(menu, song);
+                }
+                if(path)                                                                
+                    gtk_tree_path_free(path);                                               
+            }
+        }
+
+
+
 		/* add the delete widget */
 		item = gtk_image_menu_item_new_from_stock(GTK_STOCK_REMOVE,NULL);
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_find3_browser_delete_selected_songs), NULL);
-		/* add sub menu */
-		if(gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection(GTK_TREE_VIEW(pl3_find3_tree))) == 1)
-		{
-			mpd_Song *song = NULL;
-			GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(pl3_find3_tree));
-			GtkTreePath *path;
-			GtkTreeIter iter;
-			GList *list = gtk_tree_selection_get_selected_rows(gtk_tree_view_get_selection(GTK_TREE_VIEW(pl3_find3_tree)),&model);
-			path = list->data;
-			/* free result */
-			g_list_free(list);
-			if(path && gtk_tree_model_get_iter(model, &iter, path))
-			{
-				gtk_tree_model_get(model, &iter, MPDDATA_MODEL_COL_MPDSONG, &song, -1);
-				if(song)
-					submenu_for_song(menu, song);
-			}
-			if(path)
-				gtk_tree_path_free(path);
-
-		}
-
-
+	
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu),gtk_separator_menu_item_new());
 
 		if(mpd_server_check_version(connection,0,12,0))
@@ -641,7 +799,7 @@ static gboolean pl3_find3_browser_button_release_event(GtkWidget *but, GdkEventB
         G_CALLBACK(pl3_find3_browser_edit_columns), NULL);
 
     gtk_widget_show_all(menu);
-    gtk_menu_popup(GTK_MENU(menu), NULL, NULL,NULL, NULL, event->button, event->time);
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL,NULL, NULL, 0, event->time);
     return TRUE;
   }
   return FALSE ;
@@ -649,6 +807,7 @@ static gboolean pl3_find3_browser_button_release_event(GtkWidget *but, GdkEventB
 
 static void pl3_find3_browser_disconnect()
 {
+/*   while(criterias3)pl3_find3_browser_remove_crit(NULL, criterias3);*/
   /*	if(pl3_find3_store) gtk_list_store_clear(pl3_find3_store);*/
 }
 
@@ -695,6 +854,7 @@ static int pl3_find3_browser_add_go_menu(GtkWidget *menu)
   item = gtk_image_menu_item_new_with_label(_("Playlist Search"));
   gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
       gtk_image_new_from_icon_name("playlist-search-browser", GTK_ICON_SIZE_MENU));
+  gtk_widget_add_accelerator(GTK_WIDGET(item), "activate", gtk_menu_get_accel_group(GTK_MENU(menu)), GDK_F2, 0, GTK_ACCEL_VISIBLE);
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_find3_browser_activate), NULL);
 
@@ -711,12 +871,13 @@ static void pl3_find3_browser_connection_changed(MpdObj *mi, int connect, gpoint
 
 static int pl3_find3_browser_key_press_event(GtkWidget *mw, GdkEventKey *event, int type)
 {
+/*
   if (event->keyval == GDK_F2)
   {
     pl3_find3_browser_activate();
     return TRUE;
   }                                           	
-  else if(event->state&GDK_CONTROL_MASK && event->keyval == GDK_j)
+  else*/ if(event->state&GDK_CONTROL_MASK && event->keyval == GDK_j)
   {
     pl3_playlist_search();
     crit3_struct *cs;
@@ -804,18 +965,34 @@ static void pl3_find3_browser_delete_selected_songs (void)
 
 static void pl3_find3_browser_destroy(void)
 {
-  if(pl3_find3_vbox)
-  {
-    gtk_widget_destroy(pl3_find3_vbox);
-  }
-  if(pl3_find3_store2)
-  {
-    g_object_unref(pl3_find3_store2);
-  }
-  if(pl3_find3_ref)
-  {
-    gtk_tree_row_reference_free(pl3_find3_ref);
-  }
+    if(pl3_find3_ref)
+    {
+        GtkTreeModel *model = gtk_tree_row_reference_get_model(pl3_find3_ref);
+        GtkTreePath *path = gtk_tree_row_reference_get_path(pl3_find3_ref);
+        if(path)
+        {
+            GtkTreeIter iter;
+            if(gtk_tree_model_get_iter(model, &iter, path))
+            {
+                gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
+            }
+            gtk_tree_path_free(path);
+        }
+        gtk_tree_row_reference_free(pl3_find3_ref);
+        pl3_find3_ref = NULL;
+    }
+
+    if(pl3_find3_vbox)
+    {
+        gtk_widget_destroy(pl3_find3_vbox);
+        pl3_find3_vbox = NULL;
+    }
+    if(pl3_find3_store2)
+    {
+        g_object_unref(pl3_find3_store2);
+        pl3_find3_store2 = NULL;
+    }
+
 }
 static void pl3_find3_browser_status_changed(MpdObj *mi,ChangedStatusType what, void *data)
 {
@@ -824,4 +1001,18 @@ static void pl3_find3_browser_status_changed(MpdObj *mi,ChangedStatusType what,
     pl3_find3_browser_search(); 
   }
 }	
+static void pl3_find3_save_myself(void)
+{
+	if(pl3_find3_ref)
+	{
+		GtkTreePath *path = gtk_tree_row_reference_get_path(pl3_find3_ref);
+		if(path)
+		{
+			gint *indices = gtk_tree_path_get_indices(path);
+			debug_printf(DEBUG_INFO,"Saving myself to position: %i\n", indices[0]);
+			cfg_set_single_value_as_int(config, "find3-browser","position",indices[0]);
+			gtk_tree_path_free(path);
+		}
+	}
+}
 
