@@ -145,7 +145,6 @@ static void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
 				new->type = TYPE_ITEM;
 				new->name = g_strndup(buffer, len);	
 				cfgo->total_size+= len+sizeof(config_node);
-			/*	printf("node size: %i\n", sizeof(config_node));*/
 				/* Get value */
 				len = 0;
 				/* skip spaces */
@@ -873,7 +872,7 @@ conf_mult_obj *cfg_get_key_list(config_obj *data,char *class)
 
 static void __int_cfg_do_special_cleanup(config_obj *cfg, config_node *node)
 {
-	config_node *root = NULL;
+	config_node *item,*root = NULL;
 	if(!cfg || !cfg->root)
 		return;
 	if(node == NULL)
@@ -887,10 +886,20 @@ static void __int_cfg_do_special_cleanup(config_obj *cfg, config_node *node)
 		{
 			if(root->children)
 				__int_cfg_do_special_cleanup(cfg, root->children);		
+			if(root->children == NULL)
+			{
+				item = NULL;
+				if(root->prev)
+					item =  root->prev;
+				else if(root->next)
+					item = root->next;
+				__int_cfg_remove_node(cfg, root);	
+				root = item;
+			}
 		}	
 		else if(root->type == TYPE_ITEM)
 		{
-			if(root->value == NULL || root->value[0] == '\0')
+			if(root->value == NULL || root->value[0] == '\0' || strlen(root->value) == 0 )
 			{
 				config_node *node = root;
 				root = node->prev;
@@ -900,7 +909,8 @@ static void __int_cfg_do_special_cleanup(config_obj *cfg, config_node *node)
 				if(!root) return;
 			}
 		}
-		root = root->next;
+        if(root)
+            root = root->next;
 	}
 }
 void cfg_do_special_cleanup(config_obj *cfg)
