@@ -43,11 +43,12 @@
 #include "browsers/playlist3-file-browser.h"
 #include "browsers/playlist3-find2-browser.h"
 #include "browsers/playlist3-find3-browser.h"
-#include "browsers/playlist3-tag-browser.h"
+#include "browsers/playlist3-tag2-browser.h"
+#ifdef ARTIST_BROWSER
 #include "browsers/playlist3-artist-browser.h"
+#endif
 #include "browsers/playlist3-current-playlist-browser.h"
-
-#include "id3info.h"
+#include "misc.h"
 
 /**
  * Get revision
@@ -65,12 +66,21 @@
 #include "bacon/bacon-message-connection.h"
 static BaconMessageConnection *bacon_connection = NULL;
 #endif
-
+extern GladeXML *pl3_xml;
+/**
+ * Global objects that give signals
+ */
+/* gives signal on connection changes, and state changes of mpd.*/
 GmpcConnection *gmpcconn = NULL;
+/* Implements, and gives signals on profiles */
 GmpcProfiles *gmpc_profiles = NULL;
+GmpcSignals *gmpc_signals =NULL;
+/* Implements, and gives signals on meta_data*/
+GmpcMetaWatcher *gmw = NULL;
+/* the state the user set gmpc in, so if the user told disconnect, don't try to autoconnect again.. */
 int gmpc_connected = FALSE;
-int gmpc_failed_tries = 0;
-
+static void connection_changed_real(GmpcConnection *gmpcconn,MpdObj *mi, int connect,gpointer data);
+static void gmpc_status_changed_callback_real(GmpcConnection *gmpcconn, MpdObj *mi, ChangedStatusType what, gpointer data);
 
 /**
  * Define some local functions
@@ -102,9 +112,9 @@ GladeXML *xml_password_window = NULL;
 static int autoconnect_callback (void);
 
 /*
- * the ID of the autoconnect timeout
+ * the ID of the autoconnect timeout callback
  */
-guint autoconnect_timeout = 0;
+static guint autoconnect_timeout = 0;
 
 /*
  * The Config object
@@ -121,73 +131,17 @@ void connect_callback(MpdObj *mi);
 /* Glade prototypes, these would be static otherwise */
 void send_password(void);
 
+/* hack */
+extern guint sel_changed_handler_id;
 /**
  * Set paths
  */
 static void create_gmpc_paths(void);
 
-
-/**
- * Get's the full path to an image,
- * While this is compile time on linux, windows
- * needs to determine it run-time.
- */
-char *gmpc_get_full_image_path(char *filename)
-{
-    gchar *path;
-#ifdef WIN32
-    gchar *packagedir;
-    packagedir = g_win32_get_package_installation_directory("gmpc", NULL);
-    debug_printf(DEBUG_INFO, "Got %s as package installation dir", packagedir);
-
-    path = g_build_filename(packagedir, "data", "images", filename, NULL);
-
-    /* From a certain version of GTK+ this g_free will be needed, but for now it will free
-     * a pointer which is returned on further calls to g_win32_get...
-     * This bug is fixed now (30-10-2007), so it will probably be in glib 2.6.7 and/or 2.8.4
-     */
-#if GLIB_CHECK_VERSION(2,8,4)
-    q_free(packagedir);
-#endif
-
-#else
-    path = g_strdup_printf("%s/%s", PIXMAP_PATH, filename);
-#endif
-    return path;
-}
-
-/** 
- * Get the full path to the glade files.
- * While this is compile time on linux, windows 
- * needs to determine it run-time
- */
-char *gmpc_get_full_glade_path(char *filename)
-{
-    gchar *path;
-#ifdef WIN32
-    gchar *packagedir;
-    packagedir = g_win32_get_package_installation_directory("gmpc", NULL);
-    debug_printf(DEBUG_INFO, "Got %s as package installation dir", packagedir);
-
-    path = g_build_filename(packagedir, "data", "glade", filename, NULL);
-
-    /* From a certain version of GTK+ this g_free will be needed, but for now it will free
-     * a pointer which is returned on further calls to g_win32_get...
-     * This bug is fixed now (30-10-2007), so it will probably be in glib 2.6.7 and/or 2.8.4
-     */
-#if GLIB_CHECK_VERSION(2,8,4)
-    q_free(packagedir);
-#endif
-
-#else
-    path = g_strdup_printf("%s/%s", GLADE_PATH, filename);
-#endif
-    return path;
-}
 #ifndef WIN32
 static void bacon_on_message_received(const char *message, gpointer data)
 {
-    pl3_show_window();
+   create_playlist3(); 
 }
 #endif
 
@@ -196,6 +150,8 @@ int main (int argc, char **argv)
     int i;
     int clean_config = FALSE;
     char *config_path = NULL;
+    int start_hidden = FALSE;
+    int load_plugins = TRUE;
 #ifdef WIN32
 	gchar *packagedir;
 #endif
@@ -229,6 +185,7 @@ int main (int argc, char **argv)
     textdomain (GETTEXT_PACKAGE);
 
 #endif
+    gtk_set_locale();
     /**
      * Parse Command line options
      */
@@ -254,11 +211,11 @@ int main (int argc, char **argv)
              */
             else if (!strcasecmp(argv[i], _("--version")))
             {
-                printf(("Gnome Music Player Client\n"));
-                printf(_("Version:\t%s\n"), VERSION);
+                printf(_("Gnome Music Player Client\n"));
+                printf("%s:\t%s\n",_("Version"), VERSION);
                 if(revision && revision[0] != '\0')
                 {
-                    printf(_("Revision:\t%s\n"),revision);
+                    printf("%s:\t%s\n",_("Revision"),revision);
                 }
                 exit(0);
             }
@@ -269,27 +226,36 @@ int main (int argc, char **argv)
             {
                 config_path = g_strdup(&argv[i][strlen(_("--config="))]);
             }
+            else if (!strncasecmp(argv[i], _("--start-hidden"), strlen(_("--start-hidden"))))
+            {
+                start_hidden = TRUE;
+            }
             else if (!strncasecmp(argv[i], _("--clean-cover-db"), strlen(_("--clean-cover-db"))))
             {
                 clean_config = TRUE;
             }
-            /**
+            else if (!strncasecmp(argv[i], _("--disable-plugins"), strlen(_("--disable-plugins"))))
+            {
+                load_plugins = FALSE;
+            }                                                                                                /**
              * Print out help message
              */
             else if (!strncasecmp(argv[i], _("--help"),strlen(_("--help"))))
             {
                 printf(_("Gnome Music Player Client\n"\
                             "Options:\n"\
-                            "\t--help:\t\t\tThis help message.\n"\
+                            "\t--start-hidden\t\tStart hidden\n"\
+                            "\t--help\t\t\tThis help message.\n"\
                             "\t--debug-level=<level>\tMake gmpc print out debug information.\n"\
                             "\t\t\t\tLevel:\n"\
-                            "\t\t\t\t\t0: No Output\n"\
-                            "\t\t\t\t\t1: Error Messages\n"\
-                            "\t\t\t\t\t2: Error + Warning Messages\n"\
-                            "\t\t\t\t\t3: All messages\n"\
-                            "\t--version:\t\tPrint version and svn revision\n"\
+                            "\t\t\t\t\t0 No Output\n"\
+                            "\t\t\t\t\t1 Error Messages\n"\
+                            "\t\t\t\t\t2 Error + Warning Messages\n"\
+                            "\t\t\t\t\t3 All messages\n"\
+                            "\t--version\t\tPrint version and svn revision\n"\
                             "\t--config=<file>\t\tSet config file path, default  ~/.gmpc/gmpc.cfg\n"\
-                            "\t--clean-cover-db\tCleanup the cover file.\n"
+                            "\t--clean-cover-db\tCleanup the cover file.\n"\
+                            "\t--disable-plugins\tDon't load any plugins.\n"
                         ));
                 exit(0);
             }
@@ -338,10 +304,6 @@ int main (int argc, char **argv)
         }	
 
     }	
-
-
-
-
     create_gmpc_paths();
 
     /* do the clean config stuff */
@@ -362,11 +324,9 @@ int main (int argc, char **argv)
      * Check if the user has forced a different config file location.
      * else set to ~/.gmpc/gmpc.cfg
      */
-    if(!config_path)
-    {
-        url = g_strdup_printf("%s%c.gmpc%cgmpc.cfg", g_get_home_dir(),G_DIR_SEPARATOR,G_DIR_SEPARATOR);
-    }
-    else{
+    if(!config_path) {
+        url = gmpc_get_user_path("gmpc.cfg");
+    } else {
         url = config_path;
     }
     /**
@@ -389,26 +349,68 @@ int main (int argc, char **argv)
      * cleanup 
      */
     q_free(url);
-
+    /** 
+     * If requested, output debug info to file 
+     */
+    if(cfg_get_single_value_as_int_with_default(config, "Default", "Debug-log",FALSE))
+    {
+        url = gmpc_get_user_path("debug-info.log");
+        if(url)
+        {
+            FILE *fp = fopen(url, "a");
+            if(!fp)
+            {
+                debug_printf(DEBUG_ERROR, "Failed to open debug-log file: \"%s\"\n", url);
+                show_error_message(_("Failed to load debug-log file."), TRUE);
+                abort();
+            }
+            /* Set the output */
+            debug_set_output(fp);
+            /* Force highest level */
+            debug_set_level(DEBUG_INFO);
+            debug_printf(DEBUG_INFO,"***** Opened debug log file\n");
+            q_free(url);
+        }
+    }
 	/**
 	 * TODO, Check if version changed, then say something about it 
 	 */
 	url = cfg_get_single_value_as_string(config, "Default", "version");
 	if(url == NULL || strcmp(url, VERSION))
 	{
-		printf("Welcome to a new version of gmpc.\n");
-		/* Do possible cleanup of config files and stuff */
+        int *new_version = split_version(VERSION); 
+        if(url)
+        {
+            int *old_version = split_version((const char *)url);
+            debug_printf(DEBUG_INFO,"Welcome to a new version of gmpc.\n");
+            /* Do possible cleanup of config files and stuff */
+            if(!(old_version[0] >= 0 && old_version[1] >= 15 && old_version[2] >= 4 && old_version[3] >= 98))
+            {
+                conf_mult_obj *iter,*cmo = cfg_get_class_list(config);
+                debug_printf(DEBUG_INFO,"Purging old keys from the config file.\n");
+                for(iter = cmo; iter ; iter = iter->next)
+                {
+                    if(strstr(iter->key, "colpos") || strstr(iter->key, "colshow") || strstr(iter->key, "colsize"))
+                    {
+                        debug_printf(DEBUG_INFO,"Removing entry: %s\n", iter->key);
+                        cfg_remove_class(config, iter->key);
+                    }
+                }
+                cfg_free_multiple(cmo);
 
+
+            }
+            q_free(old_version);
+        }
 		/* set new */
 		cfg_set_single_value_as_string(config, "Default", "version",VERSION); 
+        q_free(new_version);
 	}
 	if(url){
 		q_free(url);
 	}
 
 
-    gmpc_profiles = gmpc_profiles_new();
-
 
 #ifndef WIN32
 
@@ -424,22 +426,29 @@ int main (int argc, char **argv)
             {
                 debug_printf(DEBUG_WARNING, "gmpc is allready running\n");
                 bacon_message_connection_send(bacon_connection, "PRESENT");
-                exit(0);
+				bacon_message_connection_free (bacon_connection);
+				cfg_close(config);
+				config = NULL;
+				exit(0);
             }
             bacon_message_connection_set_callback (bacon_connection,
                     bacon_on_message_received,
                     NULL);
         }
     }
-
-
 #endif		
+
+
     /**
      * Setup session support
      */
 #ifdef ENABLE_SM
     smc_connect(argc, argv);
 #endif	
+    /** Signals */
+    gmpc_signals = gmpc_signals_new();
+
+    gmpc_profiles = gmpc_profiles_new();
     /**
      * Initialize the new metadata subsystem.
      * (Will spawn a new thread, so have to be after the init threading 
@@ -461,7 +470,7 @@ int main (int argc, char **argv)
         /**
          * if failed, print error message
          */
-        debug_printf(DEBUG_ERROR,"Failed to create connection obj\n");
+        debug_printf(DEBUG_ERROR,"Failed to create connection object\n");
         show_error_message(_("Failed to setup libmpd"), TRUE);
         abort();
     }
@@ -476,6 +485,15 @@ int main (int argc, char **argv)
      * Just some trick to provide glib signals
      */
     gmpcconn = (GmpcConnection *)gmpc_connection_new();
+    g_signal_connect(G_OBJECT(gmpcconn), "connection_changed", G_CALLBACK(connection_changed_real),NULL);
+    g_signal_connect(G_OBJECT(gmpcconn), "status_changed", G_CALLBACK(gmpc_status_changed_callback_real), NULL);
+	/**
+	 * New Metadata object 
+	 */
+	gmw = gmpc_meta_watcher_new();
+
+
+
 
     /** 
      * Add the internall plugins 
@@ -486,7 +504,9 @@ int main (int argc, char **argv)
     /** file browser */
     plugin_add(&file_browser_plug, 0);
     /** Artist browser */
+#ifdef ARTIST_BROWSER
     plugin_add(&artist_browser_plug, 0);
+#endif
     /** File Browser */
     plugin_add(&find2_browser_plug, 0);
     /* this shows the connection preferences */
@@ -496,18 +516,23 @@ int main (int argc, char **argv)
     /* this shows the playlist preferences */
     plugin_add(&playlist_plug,0);
     /* this shows the markup stuff */
-    plugin_add(&tag_plug,0);
+    plugin_add(&tag2_plug,0);
 #ifdef ENABLE_MMKEYS
     plugin_add(&mmkeys_plug,0);
 #endif
     plugin_add(&url_plugin,0);
     /* the tray icon */
-    plugin_add(&tray_icon_plug,0);
+    /*plugin_add(&tray_icon_plug,0);*/
+    plugin_add(&tray_icon2_plug,0);
     /* Info3 data browser */
-    plugin_add(&info3_plugin,0);	
     /* Meta data browser */
     plugin_add(&metab_plugin,0);	
+	/* Playlist editor */
+	plugin_add(&playlist_editor_plugin,0);
 
+	plugin_add(&proxyplug,0);
+
+	plugin_add(&pl3_cat_editor_plug, 0);
 
 
     /**
@@ -517,29 +542,26 @@ int main (int argc, char **argv)
 	packagedir = g_win32_get_package_installation_directory("gmpc", NULL);
     debug_printf(DEBUG_INFO, "Got %s as package installation dir", packagedir);
     url = g_build_filename(packagedir, "data", "plugins", NULL);
-	/* From a certain version of GTK+ this g_free will be needed, but for now it will free
-	 * a pointer which is returned on further calls to g_win32_get...
-	 * This bug is fixed now (30-10-2007), so it will probably be in glib 2.6.7 and/or 2.8.4
-	 */
-#if GLIB_CHECK_VERSION(2,8,4)
 	q_free(packagedir);
-#endif
-
 
 #else
 	/** Load the global installed plugins */
-	url = g_strdup_printf("%s%c%s",GLADE_PATH,G_DIR_SEPARATOR, "plugins");
+	url = g_build_path(G_DIR_SEPARATOR_S, GLADE_PATH,"plugins",NULL);
 #endif
-	plugin_load_dir(url);
+    if(load_plugins)
+        plugin_load_dir(url);
 	q_free(url);
+
 	/* user space dynamic plugins */
-	url = g_strdup_printf("%s%c.gmpc%cplugins%c",g_get_home_dir(),G_DIR_SEPARATOR,G_DIR_SEPARATOR,G_DIR_SEPARATOR);
+	url = gmpc_get_user_path("plugins");
 	/**
 	 * if dir exists, try to load the plugins.
 	 */
 	if(g_file_test(url, G_FILE_TEST_IS_DIR))
 	{
-		plugin_load_dir(url);
+		debug_printf(DEBUG_INFO, "Trying to load plugins in: %s", url);
+        if(load_plugins)
+            plugin_load_dir(url);
 	}
 	q_free(url);
 
@@ -567,6 +589,8 @@ int main (int argc, char **argv)
 	 */	
 	debug_printf(DEBUG_INFO, "Create main window\n");
 	create_playlist3();
+    playlist3_message_init();
+
 	/** 
 	 * If autoconnect is enabled, tell gmpc that it's in state it should connect
 	 */	
@@ -585,12 +609,17 @@ int main (int argc, char **argv)
 	 * create the autoconnect timeout, if autoconnect enable, it will check every 5 seconds
 	 * if you are still connected, and reconnects you if not.
 	 */
-	autoconnect_timeout = g_timeout_add (5000,(GSourceFunc)autoconnect_callback, NULL);
-	/** 
+    autoconnect_timeout = g_timeout_add (5000,(GSourceFunc)autoconnect_callback, NULL);
+    /** 
 	 * Call this when entering the main loop, so you are connected on startup, not 5 seconds later
 	 */
 	gtk_init_add((GSourceFunc)autoconnect_callback, NULL);
 
+    if(cfg_get_single_value_as_int_with_default(config, "Default", "start-hidden", FALSE) || start_hidden)
+    {
+        gtk_init_add((GSourceFunc)pl3_hide, NULL);
+    }
+
 #ifdef ENABLE_MMKEYS
 	/**
 	 * Setup Multimedia Keys
@@ -610,12 +639,12 @@ int main (int argc, char **argv)
 	g_signal_connect(G_OBJECT(keys), "mm_fastbackward", G_CALLBACK(song_fastbackward), NULL);
 	g_signal_connect(G_OBJECT(keys), "mm_repeat", G_CALLBACK(repeat_toggle), NULL);
 	g_signal_connect(G_OBJECT(keys), "mm_random", G_CALLBACK(random_toggle), NULL);
-	g_signal_connect(G_OBJECT(keys), "mm_raise", G_CALLBACK(pl3_show_window), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_raise", G_CALLBACK(create_playlist3), NULL);
 	g_signal_connect(G_OBJECT(keys), "mm_hide", G_CALLBACK(pl3_hide), NULL);
 	g_signal_connect(G_OBJECT(keys), "mm_toggle_hidden", G_CALLBACK(pl3_toggle_hidden), NULL);
 	g_signal_connect(G_OBJECT(keys), "mm_volume_up", G_CALLBACK(volume_up), NULL);
 	g_signal_connect(G_OBJECT(keys), "mm_volume_down", G_CALLBACK(volume_down), NULL);
-	g_signal_connect(G_OBJECT(keys), "mm_show_notification", G_CALLBACK(tray_notify_popup), NULL );
+	g_signal_connect(G_OBJECT(keys), "mm_show_notification", G_CALLBACK(tray_icon2_create_tooltip), NULL );
 
 
 #endif
@@ -639,12 +668,24 @@ int main (int argc, char **argv)
 	 * Clear metadata struct
 	 */
 	meta_data_destroy();
+	g_object_unref(G_OBJECT(gmw));
 
+
+	/* tell the plugins to save themself. */
+	for(i=0; i< num_plugins && plugins[i] != NULL;i++) {
+		if(plugins[i]->save_yourself) {
+			debug_printf(DEBUG_INFO,"Telling '%s' to save itself\n", plugins[i]->name);
+			plugins[i]->save_yourself();
+		}
+	}
+    /* Should fix some possible crashes */
+    g_signal_handler_disconnect(gtk_tree_view_get_selection(playlist3_get_category_tree_view()), sel_changed_handler_id);
+    gtk_tree_view_set_model(playlist3_get_category_tree_view(),NULL);
+
+    
 	/* time todo some destruction of plugins */
-	for(i=0; i< num_plugins && plugins[i] != NULL;i++)
-	{
-		if(plugins[i]->destroy)
-		{
+	for(i=0; i< num_plugins && plugins[i] != NULL;i++) {
+		if(plugins[i]->destroy) {
 			plugins[i]->destroy();
 		}
 	}
@@ -660,6 +701,7 @@ int main (int argc, char **argv)
 	 */
 	mpd_free(connection);
 
+    playlist3_message_destroy();
 	/**
 	 * remove (probly allready done) 
 	 * the playlist object
@@ -667,6 +709,8 @@ int main (int argc, char **argv)
 	g_object_unref(playlist);
 	/* cleanup curl */
 	curl_global_cleanup();
+
+    debug_printf(DEBUG_INFO, "Quit....\n");
 	return 0;
 }
 
@@ -683,11 +727,8 @@ void main_quit()
 	/**
 	 * Remove the autoconnect timeout,
 	 */
-	g_source_remove(autoconnect_timeout);
-	/**
-	 * destroy the current playlist..
-	 */
-	pl3_current_playlist_destroy();
+	if(autoconnect_timeout)
+        g_source_remove(autoconnect_timeout);
 
 	/** 
 	 * Call the connection changed.
@@ -721,15 +762,9 @@ static int autoconnect_callback(void)
 		/*
 		 * connect when autoconnect is enabled, the user wants to be connected, and it hasn't failed 3 times 
 		 */
-		if (gmpc_failed_tries <  cfg_get_single_value_as_int_with_default(config, "connection","number-of-retries", 3) 
-				&& gmpc_connected && cfg_get_single_value_as_int_with_default(config,"connection","autoconnect",FALSE))
+		if ( gmpc_connected && cfg_get_single_value_as_int_with_default(config, "connection","autoconnect", DEFAULT_AUTOCONNECT))
 		{
-			/** updated failed time, if it doesn't fail it will be set to 0
-			 * later 
-			 */
-			gmpc_failed_tries++;
 			connect_to_mpd ();
-
 		}
 	}
 	/**
@@ -754,7 +789,7 @@ static void init_stock_icons()
 	g_object_unref(G_OBJECT(pb));
 
 #ifdef WIN32
-	/* hack to help files */
+	/* hack to help finding files */
 	gchar *hack = NULL;
 	path = gmpc_get_full_image_path("");
 	hack = g_strdup_printf("%s%chicolor%c32x32%cactions",path, G_DIR_SEPARATOR,G_DIR_SEPARATOR, G_DIR_SEPARATOR);
@@ -795,23 +830,22 @@ static void init_playlist_store ()
  * This mostly involves propegating the signal
  */
 void   GmpcStatusChangedCallback(MpdObj *mi, ChangedStatusType what, void *userdata)
+{
+	gmpc_connection_status_changed(gmpcconn, mi, what);
+}
+
+static void gmpc_status_changed_callback_real(GmpcConnection *gmpcconn, MpdObj *mi, ChangedStatusType what, gpointer data)
 {
 	int i;
-	/**
-	 * Propagete to the id3 window, so it can update time
-	 */
-	id3_status_update();
 	/**
 	 * Make the plugins recieve the signals 
 	 */
-	for(i=0; i< num_plugins; i++)
-	{
-		if(plugins[i]->mpd_status_changed!= NULL)
-		{
+	for(i=0; i< num_plugins; i++) {
+		if(plugins[i]->mpd_status_changed!= NULL) {
 			plugins[i]->mpd_status_changed(mi,what,NULL);
 		}
 	}
-	gmpc_connection_status_changed(gmpcconn, mi, what);
+
 }
 
 
@@ -832,11 +866,13 @@ static void error_window_destroy(GtkWidget *window,int response, gpointer autoco
 
 static void password_dialog(int failed)
 {
-	gchar *path  = NULL;
+    GtkWidget *pl3_win = glade_xml_get_widget(pl3_xml, "pl3_win");
+    gchar *path  = NULL;
 	if(xml_password_window) return;
 	path = gmpc_get_full_glade_path("gmpc.glade");
 	xml_password_window = glade_xml_new(path, "password-dialog",NULL);
-	q_free(path);
+    gtk_window_set_transient_for(GTK_WINDOW(glade_xml_get_widget(xml_password_window, "password-dialog")), GTK_WINDOW(pl3_win));
+    q_free(path);
 	if(!xml_password_window) return;
 	if(failed)
 	{
@@ -884,26 +920,16 @@ static void error_callback(MpdObj *mi, int error_id, char *error_msg, gpointer d
 	/* if we are not connected we show a reconnect */
 	if(!mpd_check_connected(mi))
 	{
+		GtkWidget *button;
 		/* no response? then we just ignore it when autoconnecting. */
 		if(error_id == 15 && autoconnect) return;
-		if (xml_error_window == NULL)
-		{
-			gchar *str = g_strdup_printf(_("error code %i: %s"), error_id, error_msg);
-			gchar *path = gmpc_get_full_glade_path("gmpc.glade");
-			xml_error_window = glade_xml_new(path,"error_dialog",NULL);
-			q_free(path);
-			GtkWidget *dialog = glade_xml_get_widget(xml_error_window, "error_dialog");
-			gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml_error_window,"em_label")), str);
-			gtk_widget_show_all(dialog);
-			g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(error_window_destroy), GINT_TO_POINTER(autoconnect));
-			q_free(str);
-		}
-		else
-		{
-			gchar *str = g_strdup_printf(_("error code %i: %s"), error_id, error_msg);
-			gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml_error_window,"em_label")), str);
-			q_free(str);
-		}
+
+		gchar *str = g_markup_printf_escaped("<b>%s %i: %s</b>",_("error code"), error_id, error_msg);
+		playlist3_show_error_message(str, ERROR_CRITICAL);
+		button = gtk_button_new_from_stock(GTK_STOCK_CONNECT);
+		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(connect_to_mpd), NULL);
+		playlist3_error_add_widget(button);
+		g_free(str);
 	}
 	else
 	{
@@ -921,9 +947,9 @@ static void error_callback(MpdObj *mi, int error_id, char *error_msg, gpointer d
 			password_dialog(FALSE);
 		}
 		else {
-			gchar *str = g_strdup_printf(_("The following error occured: %i:'%s'"), error_id, error_msg);
-			show_error_message(str, FALSE);
-			q_free(str);
+			gchar *str = g_markup_printf_escaped("<b>%s %i: %s</b>",_("error code"), error_id, error_msg);
+			playlist3_show_error_message(str, ERROR_CRITICAL);
+			g_free(str);
 		}
 	}
 }
@@ -938,8 +964,9 @@ void connect_callback(MpdObj *mi)
 				DEFAULT_AUTOCONNECT);
 		error_window_destroy(glade_xml_get_widget(xml_error_window, "error_dialog"),0,
 				GINT_TO_POINTER(autocon));
+		
 	}
-
+	playlist3_close_error();
 }
 
 /**
@@ -947,54 +974,76 @@ void connect_callback(MpdObj *mi)
  */
 static void connection_changed(MpdObj *mi, int connect, gpointer data)
 {
-	int i=0;
-	/**
-	 * send password, first thing we do, if connected 
-	 */
-	if(connect)
-	{
-		/* set failed to 0 */
-		gmpc_failed_tries = 0;		
-		if(connection_use_auth())
-		{
-			mpd_send_password(connection);
-		}
-	}	
+    /* propagate the signal to the connection object */
+    gmpc_connection_connection_changed(gmpcconn, mi, connect);
+}
 
-	/**
-	 * propegate signals
-	 */
-	debug_printf(DEBUG_INFO, "Connection changed\n");
-	playlist_connection_changed(mi, connect);
-#ifdef ENABLE_TRAYICON
-	tray_icon_connection_changed(mi, connect);
-#endif
-	for(i=0; i< num_plugins; i++)
-	{
-		debug_printf(DEBUG_INFO, "Connection changed plugin: %s\n", plugins[i]->name);
-		if(plugins[i]->mpd_connection_changed!= NULL)
-		{
-			plugins[i]->mpd_connection_changed(mi,connect,NULL);
-		}
-	}
-	gmpc_connection_connection_changed(gmpcconn, mi, connect);
-	/**
-	 * force an update of status
-	 */
-	mpd_status_update(mi);
+static void connection_changed_real(GmpcConnection *gmpcconn,MpdObj *mi, int connect, gpointer data)
+{
+    int i=0;
 
+    if(connect)
+    {
+        if(autoconnect_timeout)
+            g_source_remove(autoconnect_timeout);
+        autoconnect_timeout = 0;
 
-	if(connect && cfg_get_single_value_as_int_with_default(config, "connection", "warning", TRUE) &&
-			mpd_check_connected(connection))
-	{
-		if(!mpd_server_check_version(connection, 0,12,0)) {
-			GtkWidget *dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
-					GTK_MESSAGE_WARNING,GTK_BUTTONS_CLOSE,
-					_("Gmpc is currently connected to mpd version lower then 0.12.0.\nThis might work, but is no longer supported."));
-			g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(gtk_widget_destroy), NULL);
-			gtk_widget_show(GTK_WIDGET(dialog));
-		}
-	}
+    }
+    else
+    {
+        if(autoconnect_timeout)
+            g_source_remove(autoconnect_timeout);
+        autoconnect_timeout = g_timeout_add (5000,(GSourceFunc)autoconnect_callback, NULL);
+
+    }
+
+    /**
+     * send password, first thing we do, if connected 
+     */
+    if(connect)
+    {
+        if(connection_use_auth())
+        {
+            mpd_send_password(connection);
+        }
+    }	
+
+    /**
+     * propegate signals
+     */
+    debug_printf(DEBUG_INFO, "Connection changed\n");
+    playlist_connection_changed(mi, connect);
+    for(i=0; i< num_plugins; i++)
+    {
+        debug_printf(DEBUG_INFO, "Connection changed plugin: %s\n", plugins[i]->name);
+        if(plugins[i]->mpd_connection_changed!= NULL)
+        {
+            plugins[i]->mpd_connection_changed(mi,connect,NULL);
+        }
+    }
+
+    /**
+     * force an update of status
+     */
+    mpd_status_update(mi);
+
+    if(connect && cfg_get_single_value_as_int_with_default(config, "connection", "warning", TRUE) &&
+            mpd_check_connected(connection))
+    {
+        if(!mpd_server_check_version(connection, 0,12,0)) {
+            GtkWidget *dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
+                    GTK_MESSAGE_WARNING,GTK_BUTTONS_CLOSE,
+                    _("Gmpc is currently connected to mpd version lower then 0.12.0.\nThis might work, but is no longer supported."));
+            g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(gtk_widget_destroy), NULL);
+            gtk_widget_show(GTK_WIDGET(dialog));
+        }
+    }
+    if(connect)
+    {
+        playlist3_show_error_message(_("<b>Connected to mpd</b>"), ERROR_INFO);
+    } else {
+        playlist3_show_error_message(_("<b>Disconnected from mpd</b>"), ERROR_INFO);
+    }
 }
 
 
@@ -1016,7 +1065,8 @@ void show_error_message(gchar *string, int block)
 	{    
 		GtkWidget *hbox = NULL, *image;
 		GtkWidget *vbox = NULL,*sw = NULL, *tree = NULL;
-		GtkCellRenderer *renderer;
+        GtkWidget *pl3_win = NULL;
+        GtkCellRenderer *renderer;
 		/* create dialog */
 		error_dialog = gtk_dialog_new_with_buttons(
 				_("Error occured during operation"),
@@ -1025,7 +1075,12 @@ void show_error_message(gchar *string, int block)
 				GTK_STOCK_CLOSE,
 				GTK_RESPONSE_OK,
 				NULL);
-		/** create list store */
+         if(pl3_xml)
+         {
+             pl3_win = glade_xml_get_widget(pl3_xml, "pl3_win");
+             gtk_window_set_transient_for(GTK_WINDOW(error_dialog), GTK_WINDOW(pl3_win));
+         }
+        /** create list store */
 		if(!error_list_store)
 		{
 			error_list_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
@@ -1089,18 +1144,14 @@ static void create_gmpc_paths(void)
 	 */	
 
 	/** create path */
-	gchar *url = g_strdup_printf("%s%c.gmpc%c", g_get_home_dir(),G_DIR_SEPARATOR,G_DIR_SEPARATOR);
-	debug_printf(DEBUG_INFO, "Checking for %s existence",url);
+	gchar *url = gmpc_get_user_path(NULL);
 
 	/**
 	 * Check if ~/.gmpc/ exists 
 	 * If not try to create it.
 	 */
-	if(!g_file_test(url, G_FILE_TEST_EXISTS))
-	{
-		debug_printf(DEBUG_INFO, "Trying to create %s",url);
-		if(g_mkdir(url,0700) < 0)
-		{
+	if(!g_file_test(url, G_FILE_TEST_EXISTS)) {
+		if(g_mkdir_with_parents(url,0700) < 0) {
 			debug_printf(DEBUG_ERROR, "Failed to create: %s\n", url);
 			show_error_message("Failed to create ~/.gmpc/.", TRUE);
 			abort();
@@ -1109,15 +1160,10 @@ static void create_gmpc_paths(void)
 	/**
 	 * if it exists, check if it's a directory 
 	 */
-	if (!g_file_test(url, G_FILE_TEST_IS_DIR))
-	{
-		debug_printf(DEBUG_ERROR, "%s isn't a directory.\n", url);
-		debug_printf(DEBUG_ERROR, "Quitting.\n");
+	if (!g_file_test(url, G_FILE_TEST_IS_DIR)) {
 		show_error_message("~/.gmpc/ isn't a directory.", TRUE);
 		abort();
-	}
-	else
-	{
+	} else {
 		debug_printf(DEBUG_INFO, "%s exist and is directory",url);
 	}
 	/* Free the path */
