@@ -27,16 +27,19 @@
 #include "playlist3.h"
 #include "playlist-list.h"
 #include "mpdinteraction.h"
-#include "tray-icon.h"
 #include "gmpc-profiles.h"
-
+#include "playlist3-messages.h" 
+#include "browsers/playlist3-playlist-editor.h"
 /**
  * Some gobjects
  */
 #include "gmpc-connection.h"
 #include "gmpc-metaimage.h"
+#include "gmpc-signals.h"
 
 extern GmpcConnection *gmpcconn;
+extern GmpcSignals *gmpc_signals;
+
 
 extern int gmpc_connected;
 
@@ -65,7 +68,9 @@ extern gmpcPlugin playlist_plug;
 extern gmpcPlugin cover_art_plug;
 
 extern gmpcPlugin tray_icon_plug;
-extern gmpcPlugin info3_plugin;
+extern gmpcPlugin tray_icon2_plug;
+extern gmpcPlugin proxyplug;
+//extern gmpcPlugin info3_plugin;
 extern gmpcPlugin metab_plugin;
 extern gmpcPlugin url_plugin; 
 /** main.c **/
@@ -73,6 +78,8 @@ extern GladeXML *xml_main_window;
 extern MpdObj *connection;
 extern GmpcProfiles *gmpc_profiles;
 
+extern gmpcPlugin playlist_editor_plugin;
+extern gmpcPlugin pl3_cat_editor_plug;
 
 enum{
 	TIME_FORMAT_ELAPSED,
@@ -87,7 +94,6 @@ void pl3_highlight_song_change ();
 char * edit_song_markup(char *format);
 
 gboolean playlist_filter_func(GtkTreeModel *model, GtkTreeIter *iter);
-void call_id3_window_song(mpd_Song *songstr);
 void playlist_changed(MpdObj *mi);
 
 void main_quit(void);
@@ -140,4 +146,11 @@ void preferences_show_pref_window(int plugin_id);
 void info2_activate(void);
 void info2_fill_artist_view(char *);
 void info2_fill_album_view(char *, char *);
+void info2_fill_song_view(mpd_Song *);
+/* tray stuff */
+gboolean tray_icon2_get_available(void);
+void tray_icon2_create_tooltip(void);
+
+/* usefull for some stuff */
+extern GladeXML *pl3_xml;
 #endif
