@@ -6,6 +6,8 @@
 
 #include "metadata.h"
 
+long unsigned num_queries = 0;
+
 config_obj *cover_index= NULL;
 int meta_num_plugins=0;
 gmpcPlugin **meta_plugins = NULL;
@@ -27,16 +29,20 @@ typedef struct {
 	guint id;
 	/* Data */
 	mpd_Song *song;
-	MetaDataCallback callback;
-	gpointer data;
 	MetaDataType type;
 	/* Resuls  */
 	MetaDataResult result;
 	char *result_path;
+	/* Callback */
+	MetaDataCallback callback;
+	gpointer data;
 } meta_thread_data;
 
-gboolean meta_data_handler_data_match(meta_thread_data *data, gpointer data2);
-
+gboolean meta_compare_func(meta_thread_data *mt1, meta_thread_data *mt2);
+static gboolean meta_data_handle_results(void);
+/*
+static gboolean meta_data_handler_data_match(meta_thread_data *data, gpointer data2);
+*/
 void meta_data_set_cache(mpd_Song *song, MetaDataType type, MetaDataResult result, char *path)
 {
 	if(!song) return;
@@ -83,6 +89,16 @@ void meta_data_set_cache(mpd_Song *song, MetaDataType type, MetaDataResult resul
 			}                                                                        		
 			q_free(temp);
 		}
+	} else if (type == META_ARTIST_SIMILAR) {
+		if(song->artist) {
+			char *temp = g_strdup("similar");                   		
+			if(result == META_DATA_AVAILABLE) {
+				cfg_set_single_value_as_string(cover_index, song->artist, temp,path);
+			} else {
+				cfg_set_single_value_as_string(cover_index, song->artist, temp,"");
+			}                                                                        		
+			q_free(temp);
+		}
 	} else if (type == META_SONG_TXT)	{
 		if(song->artist && song->title) {
 			char *temp = g_strdup_printf("lyrics:%s", song->title);                   		
@@ -108,7 +124,6 @@ MetaDataResult meta_data_get_from_cache(mpd_Song *song, MetaDataType type, char
 	{
 		return META_DATA_UNAVAILABLE;	
 	}
-	debug_printf(DEBUG_INFO,"request for: %s, %i\n", song->title, type);
 	/* Get values acording to type */
 	if(type == META_ALBUM_ART)
 	{
@@ -145,6 +160,30 @@ MetaDataResult meta_data_get_from_cache(mpd_Song *song, MetaDataType type, char
 
 		/* else default to fetching */
 	}
+	/* Get values acording to type */
+	else if(type == META_ARTIST_SIMILAR)
+	{
+		if(!song->artist) 
+		{
+			return META_DATA_UNAVAILABLE;	
+		}
+		*path = cfg_get_single_value_as_string(cover_index,song->artist,  "similar");
+		if(*path)
+		{
+			/* if path length is NULL, then data unavailible */
+			if(strlen(*path) == 0)
+			{
+				q_free(*path);
+
+				*path = NULL;
+				return META_DATA_UNAVAILABLE;	
+			}
+			return META_DATA_AVAILABLE;
+		}
+
+		/* else default to fetching */
+	}
+
 	else if(type == META_ALBUM_TXT)
 	{
 		gchar *temp = NULL;
@@ -207,10 +246,6 @@ MetaDataResult meta_data_get_from_cache(mpd_Song *song, MetaDataType type, char
 				*path = NULL;
 				return META_DATA_FETCHING;	
 			}
-
-
-
-
 			/* return that data is availible */
 			return META_DATA_AVAILABLE;
 		}
@@ -300,6 +335,7 @@ static void meta_data_retrieve_thread()
 		/* check if quit signal */
 		if(data->id == 0)
 		{
+            debug_printf(DEBUG_INFO, "Quit command recieved.. quitting");
 			return;
 		}
 		g_mutex_lock(meta_processing);
@@ -387,6 +423,8 @@ static void meta_data_retrieve_thread()
 		 * Push the result back
 		 */	
 		q_async_queue_push(meta_results, data);		
+        
+		g_idle_add((GSourceFunc)meta_data_handle_results,NULL);
 		/**
 		 * clear our reference to the object
 		 */
@@ -395,44 +433,15 @@ static void meta_data_retrieve_thread()
 	}while(1);
 }
 
-
-gboolean meta_data_handler_data_match(meta_thread_data *data, gpointer data2)
-{
-	if(data && data->id == GPOINTER_TO_INT(data2))
-	{
-		return 0;
-	}
-	return 1;
-}
-
-void meta_data_handle_remove_request(guint id)
+static gboolean meta_data_handle_results(void)
 {
 	meta_thread_data *data = NULL;
-	gboolean found = FALSE;
-	
-	if(!meta_commands)	
-		return;
-
-	q_async_queue_lock(meta_commands);
-	if((data = q_async_queue_remove_data_unlocked(meta_commands, (GCompareFunc)meta_data_handler_data_match, GINT_TO_POINTER(id))))
+    int test = 0;
+	if(meta_thread == g_thread_self())
 	{
-		if(data->result_path)q_free(data->result_path);
-		mpd_freeSong(data->song);
-		q_free(data);
-		found = TRUE;
+		debug_printf(DEBUG_ERROR,"Crap, handled in wrong thread\n");
+
 	}
-	q_async_queue_unlock(meta_commands);
-	/* if not found, it _could_ be that it's being processed now by the other thread, 
-	 * if that's the case push it in the queue that is checked on executing the callback.
-	 */
-	if(!found)
-		g_queue_push_head(meta_remove, GINT_TO_POINTER(id));
-}
-
-static gboolean meta_data_handle_results()
-{
-	meta_thread_data *data = NULL;
-
 	/**
 	 * Should check is one is being processed  (implemented)
 	 */
@@ -457,7 +466,14 @@ static gboolean meta_data_handle_results()
 	 */
 	for(data = q_async_queue_try_pop(meta_results);data;
 			data = q_async_queue_try_pop(meta_results)) {	
-		int test = 0, i = 0;
+		int i = 0;
+
+		gmpc_meta_watcher_data_changed(gmw,data->song, (data->type)&META_QUERY_DATA_TYPES, data->result,data->result_path);
+ 		if(data->callback)
+		{
+			data->callback(data->song,data->result,data->result_path, data->data);
+		}
+
 		for(i=g_queue_get_length(meta_remove)-1; i>=0;i--) {
 			int num = GPOINTER_TO_INT(g_queue_peek_nth(meta_remove,i));
 			if(num == data->id) { 
@@ -466,10 +482,8 @@ static gboolean meta_data_handle_results()
 				debug_printf(DEBUG_INFO, "Removing callback: %u\n", num);
 			}
 		}
-		
-		if(test == 0&& data->callback) {
-			data->callback(data->song, data->result,data->result_path, data->data);
-		}
+	
+	
 		if(data->result_path)q_free(data->result_path);
 		mpd_freeSong(data->song);
 		q_free(data);
@@ -477,7 +491,12 @@ static gboolean meta_data_handle_results()
 	/**
 	 * Keep the timer running
 	 */
-	return TRUE;
+    /* update when handled */
+    test = g_mutex_trylock(meta_processing);
+    if(test)
+        g_mutex_unlock(meta_processing);
+    gmpc_meta_watcher_queue_size_changed(gmw, q_async_queue_true_length(meta_commands)+!test, num_queries);
+    return FALSE;
 }
 
 /**
@@ -485,257 +504,225 @@ static gboolean meta_data_handle_results()
  */
 void meta_data_init()
 {
-	gchar *url = g_strdup_printf("%s/.covers/", g_get_home_dir());
-	if(!g_file_test(url,G_FILE_TEST_IS_DIR)){
-		if(g_mkdir(url, 0700)<0){
-			g_error("Cannot make %s\n", url);
-		}
-	}
-	q_free(url);
-	url = g_strdup_printf("%s/.covers/covers.db", g_get_home_dir());
-	cover_index = cfg_open(url);
-	q_free(url);
+    gchar *url = gmpc_get_covers_path(NULL); 
+    if(!g_file_test(url,G_FILE_TEST_IS_DIR)){
+        if(g_mkdir(url, 0700)<0){
+            g_error("Cannot make %s\n", url);
+        }
+    }
+    q_free(url);
+    url = gmpc_get_covers_path("covers.db");
+    cover_index = cfg_open(url);
+    q_free(url);
 
-	/**
-	 * The command queue
-	 */
-	meta_commands = q_async_queue_new();
-	/**
-	 * the result queue
-	 */
-	meta_results = q_async_queue_new();
-	/**
- 	*  remove callbacks...
- 	*  not thread save
- 	*/
-	meta_remove = g_queue_new();
+    /**
+     * The command queue
+     */
+    meta_commands = q_async_queue_new();
+    /**
+     * the result queue
+     */
+    meta_results = q_async_queue_new();
+    /**
+     *  remove callbacks...
+     *  not thread save
+     */
+    meta_remove = g_queue_new();
 
 
-	meta_processing = g_mutex_new();
-	/**
-	 * Create the retrieval thread
-	 */
-	meta_thread = g_thread_create((GThreadFunc)meta_data_retrieve_thread, NULL, TRUE, NULL);
-	/**
-	 * Set a timer on checking the results
-	 * for now every 250 ms?
-	 */
-	g_timeout_add(250,(GSourceFunc)meta_data_handle_results, NULL);
+    meta_processing = g_mutex_new();
+    /**
+     * Create the retrieval thread
+     */
+    meta_thread = g_thread_create((GThreadFunc)meta_data_retrieve_thread, NULL, TRUE, NULL);
 
 }
-/**
- * Function called by the "client" 
- */
-guint meta_data_get_path_callback(mpd_Song *tsong, MetaDataType type, MetaDataCallback callback, gpointer data)
-{
-	MetaDataResult ret;
-	meta_thread_data *mtd = NULL;
-	mpd_Song *song =NULL;
-	char *path = NULL;
-	guint id = 0;
-
-	/**
-	 * if there is no callback, it's a programming error.
-	 */
-	g_assert(callback != NULL);
-
-	/**
-	 * If there is no song
-	 * return;
-	 */
-	/*	g_return_if_fail(tsong != NULL); */
-	if(tsong == NULL)
-	{
-		return 0;	
-	}
-
-	/**
-	 * Check cache for result.
-	 */
-	if(type&META_QUERY_NO_CACHE)
-	{
-		ret = META_DATA_FETCHING;
-	}
-	else
-	{
-		ret = meta_data_get_from_cache(tsong, type&META_QUERY_DATA_TYPES, &path);
-	}
-
-	/**
-	 * If the data is know. (and doesn't need fectching) 
-	 * call the callback and stop
-	 */
-	if(ret != META_DATA_FETCHING)
-	{
-		/* Call the callback function */
-		callback(tsong, ret, path,data);
-		/* clean up path if exists */
-		if(path) q_free(path);
-		/* return */
-
-		return 0;
-	}
-
-	/**
-	 * Make a copy
-	 */
-	song = mpd_songDup(tsong);
-
-	/**
-	 * Check if the song is complete, (has file) so we can actually use it with all plugins
-	 * If not, f.e. only artist, or only album, get a song from mpd
-	 * so plugins based on path can work with it.
-	 */
-	/** For speed reason and mpd stressing disabled */
-/*	if(song->file == NULL)*/
-	/*
-	if(FALSE){
-	*/	/* Only mpd 0.12 supports this */
-	/*	if(mpd_server_check_version(connection, 0,12,0))
-		{
-			MpdData *data  = NULL;
-	*/		/** We need new libmpd data here.
-			 * The we don't need the check what type of search
-			 */
-			/**
-			 * this should be done faster, it can now cause extra slowdown
-			 * because of the mpd roundtrips
-			 */
-			/*
-			if(song->artist && song->album)
-			{
-				data= mpd_database_find_adv(connection,TRUE, 
-						MPD_TAG_ITEM_ARTIST,
-						song->artist,
-						MPD_TAG_ITEM_ALBUM,
-						song->album,
-						-1);
-			}
-			else if(song->artist)
-			{
-				data= mpd_database_find_adv(connection, TRUE,
-						MPD_TAG_ITEM_ARTIST,
-						song->artist,
-						-1);
-			}
-			if(data)
-			{
-				if(data->type == MPD_DATA_TYPE_SONG)
-				{
-					song->file = g_strdup(data->song->file);
-				}
-
-				mpd_data_free(data);
-			}
-
-		}
-	}
-*/
-	/**
-	 * If no result, start a thread and start fetching the data from there
-	 */
-
-	mtd = g_malloc0(sizeof(*mtd));
-	/**
-	 * unique id 
-	 * Not needed, but can be usefull for debugging
-	 */
-	id = mtd->id = g_random_int_range(1,2147483647);
-	mtd->song = mpd_songDup(song);
-	mtd->callback = callback;
-	mtd->data = data;
-	mtd->type = type;
-	/** push it to the other thread */
-	q_async_queue_push(meta_commands, mtd);
-	/** clean reference to pointer, it's now to the other thread */
-	mtd = NULL;
-
-	/**
-	 * Call the callback to let the client know where are going todo a 
-	 * background fetch
-	 */
-
-	/**
-	 * Tell the calling part we are fetching */
-	callback(song, META_DATA_FETCHING,NULL, data);
-	mpd_freeSong(song);
-	return id;
-}
-
 
 void meta_data_add_plugin(gmpcPlugin *plug)
 {
-	int i=0;
-	int changed = FALSE;	
-	meta_num_plugins++;
-	meta_plugins = g_realloc(meta_plugins,(meta_num_plugins+1)*sizeof(gmpcPlugin **));
-	meta_plugins[meta_num_plugins-1] = plug;
-	meta_plugins[meta_num_plugins] = NULL;
+    int i=0;
+    int changed = FALSE;	
+    meta_num_plugins++;
+    meta_plugins = g_realloc(meta_plugins,(meta_num_plugins+1)*sizeof(gmpcPlugin **));
+    meta_plugins[meta_num_plugins-1] = plug;
+    meta_plugins[meta_num_plugins] = NULL;
 
-	do{	
-		changed=0;
-		for(i=0; i< (meta_num_plugins-1);i++)
-		{
-			if(meta_plugins[i]->metadata->get_priority() > meta_plugins[i+1]->metadata->get_priority())
-			{
-				gmpcPlugin *temp = meta_plugins[i];
-				changed=1;
-				meta_plugins[i] = meta_plugins[i+1];
-				meta_plugins[i+1] = temp;
-			}
-		}
-	}while(changed);
+    do{	
+        changed=0;
+        for(i=0; i< (meta_num_plugins-1);i++)
+        {
+            if(meta_plugins[i]->metadata->get_priority() > meta_plugins[i+1]->metadata->get_priority())
+            {
+                gmpcPlugin *temp = meta_plugins[i];
+                changed=1;
+                meta_plugins[i] = meta_plugins[i+1];
+                meta_plugins[i+1] = temp;
+            }
+        }
+    }while(changed);
 }
 
 void meta_data_cleanup(void)
 {
-	cfg_do_special_cleanup(cover_index);
+    cfg_do_special_cleanup(cover_index);
 }
 
 void meta_data_check_plugin_changed()
 {
-	int old_amount= cfg_get_single_value_as_int_with_default(config, "metadata", "num_plugins", 0);
-	if(old_amount < meta_num_plugins)
-	{
-		GtkWidget *dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
-				_("A new metadata plugin was added, gmpc will now purge all missing metadata from the cache"));
-		g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(gtk_widget_destroy), NULL);
-		gtk_widget_show_all(GTK_WIDGET(dialog));
-		meta_data_cleanup();
-	}
-	if(old_amount != meta_num_plugins)
-	{
-		cfg_set_single_value_as_int(config, "metadata", "num_plugins", meta_num_plugins);
-	}
+    int old_amount= cfg_get_single_value_as_int_with_default(config, "metadata", "num_plugins", 0);
+    if(old_amount < meta_num_plugins)
+    {
+        GtkWidget *dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+                _("A new metadata plugin was added, gmpc will now purge all missing metadata from the cache"));
+        g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(gtk_widget_destroy), NULL);
+        gtk_widget_show_all(GTK_WIDGET(dialog));
+        meta_data_cleanup();
+    }
+    if(old_amount != meta_num_plugins)
+    {
+        cfg_set_single_value_as_int(config, "metadata", "num_plugins", meta_num_plugins);
+    }
 }
 
 void meta_data_destroy(void)
 {
-	meta_thread_data *mtd = NULL;
+    meta_thread_data *mtd = NULL;
 
-	if(meta_thread)
-	{
-		debug_printf(DEBUG_INFO,"Waiting for meta thread to terminate...");
-		/* remove old stuff */
-		q_async_queue_lock(meta_commands);
-		while((mtd = q_async_queue_try_pop_unlocked(meta_commands)))
-		{
-			mpd_freeSong(mtd->song);
-                        q_free(mtd);
-		}
-		/* Create the quiet signal, this is just an empty request with id 0 */
-		
-		mtd = g_malloc0(sizeof(*mtd));
-		mtd->id = 0;
-		/* push the request to the thread */
-		q_async_queue_push_unlocked(meta_commands, mtd);
-		q_async_queue_unlock(meta_commands);
-		/* wait for the thread to finish */
-		g_thread_join(meta_thread);
-		/* cleanup */
-		g_free(mtd);
-		debug_printf(DEBUG_INFO,"Done..");
-	}
-	cfg_close(cover_index);
+    if(meta_thread)
+    {
+        debug_printf(DEBUG_INFO,"Waiting for meta thread to terminate...");
+        /* remove old stuff */
+        q_async_queue_lock(meta_commands);
+        while((mtd = q_async_queue_try_pop_unlocked(meta_commands)))
+        {
+            mpd_freeSong(mtd->song);
+            q_free(mtd);
+        }
+        /* Create the quiet signal, this is just an empty request with id 0 */
+
+        mtd = g_malloc0(sizeof(*mtd));
+        mtd->id = 0;
+        /* push the request to the thread */
+        q_async_queue_push_unlocked(meta_commands, mtd);
+        q_async_queue_unlock(meta_commands);
+        /* wait for the thread to finish */
+        g_thread_join(meta_thread);
+        /* cleanup */
+        g_free(mtd);
+        debug_printf(DEBUG_INFO,"Done..");
+    }
+    if(meta_commands)
+        g_mutex_free(meta_processing);
+    if(meta_commands)
+        q_async_queue_unref(meta_commands);
+    meta_processing = NULL;
+    cfg_close(cover_index);
 }
+gboolean meta_compare_func(meta_thread_data *mt1, meta_thread_data *mt2)
+{
+    if((mt1->type&META_QUERY_DATA_TYPES) != (mt2->type&META_QUERY_DATA_TYPES))
+        return TRUE;
+    if(!gmpc_meta_watcher_match_data(mt1->type&META_QUERY_DATA_TYPES, mt1->song, mt2->song))
+    {
+        return TRUE;
+    }
+    return FALSE;
+}
+/**
+ * Function called by the "client" 
+ */
+MetaDataResult meta_data_get_path(mpd_Song *tsong, MetaDataType type, gchar **path,MetaDataCallback callback, gpointer data)
+{
+    MetaDataResult ret;
+    meta_thread_data *mtd = NULL;
+    mpd_Song *song =NULL;
+    guint id = 0;
+    int test = 0;
+    /* TODO: Validate request */
 
+    /**
+     * If there is no song
+     * return;
+     */
+    if(tsong == NULL)
+    {
+        return META_DATA_UNAVAILABLE;	
+    }
+
+    /**
+     * Check cache for result.
+     */
+    if(type&META_QUERY_NO_CACHE)
+    {
+        /* For others */
+        gmpc_meta_watcher_data_changed(gmw,tsong, (type)&META_QUERY_DATA_TYPES,META_DATA_FETCHING, NULL); 
+        if(callback)
+        {
+            callback(song,META_DATA_FETCHING,NULL,data);
+        }
+        ret = META_DATA_FETCHING;
+    }
+    else
+    {
+        ret = meta_data_get_from_cache(tsong, type&META_QUERY_DATA_TYPES, path);
+    }
+
+    /**
+     * If the data is know. (and doesn't need fectching) 
+     * call the callback and stop
+     */
+    if(ret != META_DATA_FETCHING)
+    {
+        return ret;	
+    }
+
+
+    /**
+     * Make a copy
+     */
+    song = mpd_songDup(tsong);
+    /**
+     * If no result, start a thread and start fetching the data from there
+     */
+
+    mtd = g_malloc0(sizeof(*mtd));
+    /**
+     * unique id 
+     * Not needed, but can be usefull for debugging
+     */
+    id = mtd->id = g_random_int_range(1,2147483647);
+    mtd->song = song;
+    mtd->type = type;
+    mtd->callback = callback;
+    mtd->data = data;
+    /**
+     * Check if request is allready in queue
+     */
+    /* when using old api style, the request is commited anyway */
+    if(!callback)
+    {
+        q_async_queue_lock(meta_commands);
+        if(q_async_queue_has_data(meta_commands,(GCompareFunc)meta_compare_func, mtd))
+        {
+            q_async_queue_unlock(meta_commands);
+            mpd_freeSong(song);
+            g_free(mtd);
+            return ret;
+        }
+        q_async_queue_unlock(meta_commands);
+    }
+
+    /** push it to the other thread */
+
+    num_queries ++;
+    /* I should fix this */
+    if(test)
+        g_mutex_unlock(meta_processing);
+    gmpc_meta_watcher_queue_size_changed(gmw, q_async_queue_true_length(meta_commands)+!test, num_queries);
+
+    q_async_queue_push(meta_commands, mtd);
+    /** clean reference to pointer, it's now to the other thread */
+    mtd = NULL;
+
+    return ret;
+}
