@@ -7,11 +7,10 @@
 #include "mpdinteraction.h"
 
 static int ignore = FALSE;
-extern GtkWidget *pl3_cp_tree;
+G_LOCK_DEFINE (connecting_lock);
 
 /* old stuff */
 static void preferences_update(void);
-static void disconnect_callback(MpdObj *);
 
 /* Server Settings plugin */
 static void server_pref_construct(GtkWidget *);
@@ -24,7 +23,7 @@ static void connection_pref_destroy(GtkWidget *container);
 static void ServerConnectionChangedCallback(MpdObj *mi, int connected, gpointer data);
 static void ServerStatusChangedCallback(MpdObj *mi, ChangedStatusType what, void *userdata);
 
-GladeXML *server_pref_xml = NULL;
+static GladeXML *server_pref_xml = NULL;
 gmpcPrefPlugin server_gpp = {
 	server_pref_construct,
 	server_pref_destroy
@@ -52,7 +51,7 @@ void submenu_genre_clicked(GtkWidget *item);
 void submenu_dir_clicked(GtkWidget *item);
 
 gmpcPlugin server_plug = {
-	"Server Settings", 	/** name */
+	N_("Server Settings"), 	/** name */
 	{1,1,1},			/** Version */
 	GMPC_INTERNALL,		/** Plugin Type */
 	0,					/** Internal Id */
@@ -82,7 +81,7 @@ gmpcPrefPlugin connection_gpp = {
 };
 
 gmpcPlugin connection_plug = {
-	"Connection",           /* name */
+	N_("Connection"),       /* name */
 	{1,1,1},                /* version */
 	GMPC_INTERNALL,         /* type */
 	0,                      /* id */
@@ -98,6 +97,18 @@ gmpcPlugin connection_plug = {
 	NULL                    /* set_enabled */
 };
 
+extern GladeXML *pl3_xml;
+guint connecting_pulse = 0;
+gboolean connecting_pulse_callback(void);
+gboolean connecting_pulse_callback(void)
+{
+	if(pl3_xml)
+	{
+		GtkProgressBar *pb = (GtkProgressBar *)glade_xml_get_widget(pl3_xml, "pl3_progressbar");
+		gtk_progress_bar_pulse(pb);
+	}
+	return TRUE;
+}
 /* this function doesnt use the start/stop_mpd_action because it the user doesnt want to see that */
 
 int update_mpd_status()
@@ -105,23 +116,37 @@ int update_mpd_status()
 	if(!mpd_check_connected(connection)) return TRUE;
 	mpd_status_update(connection);
 
-	/* unlock it */
 	return TRUE;
 }
 
-static void disconnect_callback(MpdObj *mi)
+static int connected_to_mpd(mpd_Connection *mpd_conn)
 {
-	/* disconnect playlist */
-	debug_printf(DEBUG_INFO, "Going To Clear the playlist-list");
-	playlist_list_clear(PLAYLIST_LIST(playlist),GTK_TREE_VIEW(pl3_cp_tree));
-	debug_printf(DEBUG_INFO, "Done Clearing the playlist-list");
-
+	gtk_widget_hide(glade_xml_get_widget(pl3_xml, "pl3_progressbar"));
+	g_source_remove(connecting_pulse);
+	connecting_pulse = 0;
+	if(connection)
+	{
+		mpd_connect_real(connection, mpd_conn);
+	}
+    G_UNLOCK(connecting_lock);
+	return FALSE;
+}
+static void connection_thread(void)
+{
+	mpd_Connection *conn = mpd_newConnection(connection_get_hostname(),connection_get_port(), DEFAULT_TIMEOUT);
+	g_idle_add((GSourceFunc)connected_to_mpd,conn);
+	return;
 }
 
 /* the functiont that connects to mpd */
 int connect_to_mpd()
 {
 	char *string = NULL;
+    if(!G_TRYLOCK(connecting_lock))
+    {
+        debug_printf(DEBUG_ERROR, "Allready busy connecting to mpd.. not doing anything");
+        return FALSE;
+    }
 	/**
 	 * Set Hostname
 	 */
@@ -145,15 +170,22 @@ int connect_to_mpd()
 	{
 		mpd_set_password(connection,"");
 	}
-
+/*
 	if(mpd_connect(connection) < 0)
 	{
 		debug_printf(DEBUG_INFO,"Connection failed\n");
 		return TRUE;
 	}
-	/* Set the title */
+*/
+	g_thread_create((GThreadFunc)connection_thread, NULL, FALSE,NULL);
+	connecting_pulse = g_timeout_add(200,(GSourceFunc)(connecting_pulse_callback),NULL);
+	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(glade_xml_get_widget(pl3_xml, "pl3_progressbar")), _("Connecting"));
+	gtk_widget_show(glade_xml_get_widget(pl3_xml, "pl3_progressbar"));
+
+	/* Set the title 
 	update_mpd_status();
 	mpd_stats_update(connection);
+	*/
 	/* set that user wants to connect */
 	gmpc_connected = TRUE;
 
@@ -167,10 +199,6 @@ static void ServerConnectionChangedCallback(MpdObj *mi, int connected, gpointer
 	{
 		connect_callback(mi);
 	}
-	else
-	{
-		disconnect_callback(mi);
-	}
 	preferences_update();
 }
 
@@ -487,7 +515,6 @@ void add_directory(const gchar *path)
 {
 	gchar *dirpath = g_path_get_dirname(path);
 	MpdData *data =  mpd_database_get_directory(connection,dirpath);
-	printf("dir: %s from path: %s\n", dirpath, path);
 	for(;data;data = mpd_data_get_next(data))
 	{
 		if(data->type == MPD_DATA_TYPE_SONG)
@@ -945,7 +972,7 @@ char *connection_get_hostname()
 {
 	gchar *profile = gmpc_profiles_get_current(gmpc_profiles);
 	gchar *retv  = gmpc_profiles_get_hostname(gmpc_profiles, profile);
-  g_free(profile);
+    g_free(profile);
 
 	return retv;
 }
@@ -998,7 +1025,7 @@ void submenu_for_song(GtkWidget *menu, mpd_Song *song)
 	smenu  = gtk_menu_new();
 	if(song->artist && song->album) 
 	{
-		/* Add all from artist */
+		/* Add all from album */
 		sitem = gtk_image_menu_item_new_with_label(_("All from album"));
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(sitem), gtk_image_new_from_icon_name("media-album", GTK_ICON_SIZE_MENU));
 		g_object_set_data_full(G_OBJECT(sitem), "artist", g_strdup(song->artist), g_free);
@@ -1010,7 +1037,7 @@ void submenu_for_song(GtkWidget *menu, mpd_Song *song)
 	}
 	if(song->artist )
 	{
-		/* Add all from album */
+		/* Add all from artist */
 		sitem = gtk_image_menu_item_new_with_label(_("All from artist"));
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(sitem), gtk_image_new_from_icon_name("media-artist", GTK_ICON_SIZE_MENU));
 		g_object_set_data_full(G_OBJECT(sitem), "artist", g_strdup(song->artist), g_free);
