@@ -41,4 +41,12 @@ enum PlaylistColums {
 	PL_COLUMN_SONGPOS,
 	PL_COLUMN_TOTAL
 };
+
+enum {
+	PLAYLIST_NO_ZOOM,
+	PLAYLIST_SMALL,
+	PLAYLIST_MINI,
+	PLAYLIST_ZOOM_LEVELS
+}PlaylistZoom;
+
 #endif
