@@ -8,7 +8,6 @@
 
 #include "main.h"
 #include "playlist3.h"
-#include "browsers/playlist3-tag-browser.h"
 #include "config1.h"
 
 #define PLUGIN_STATS -200
@@ -66,7 +65,7 @@ static void pref_plugin_changed()
 				else
 				{
 					buf = g_strdup_printf("<span size=\"xx-large\"><b>%s</b></span>",
-							plugins[id]->name);
+							_(plugins[id]->name));
 				}
 
 				plugins[id]->pref->construct(glade_xml_get_widget(xml_preferences_window, "plugin_container"));
@@ -94,6 +93,7 @@ static void pref_plugin_changed()
 
 void create_preferences_window()
 {
+	GtkWidget *pl3_win = glade_xml_get_widget(pl3_xml, "pl3_win");
 	GtkWidget *dialog;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
@@ -126,6 +126,7 @@ void create_preferences_window()
 	/* set info from struct */
 	/* hostname */
 	dialog = glade_xml_get_widget(xml_preferences_window, "preferences_window");
+    gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(pl3_win));
 	gtk_widget_show_all(GTK_WIDGET(dialog));
 	running = 1;
 
@@ -153,7 +154,7 @@ void create_preferences_window()
 				gtk_list_store_append(GTK_LIST_STORE(plugin_store), &iter);
 				gtk_list_store_set(GTK_LIST_STORE(plugin_store), &iter,
 						0, plugin_get_pos(plugins[i]->id)/*^PLUGIN_ID_MARK*/,
-						1, plugins[i]->name, -1);
+						1, _(plugins[i]->name), -1);
 				if(gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection(
 								GTK_TREE_VIEW(glade_xml_get_widget(xml_preferences_window, "plugin_tree")))) == 0)
 				{
@@ -172,7 +173,7 @@ void create_preferences_window()
 	{
 		GtkTreeIter iter;
 		gtk_list_store_append(GTK_LIST_STORE(plugin_store), &iter);
-		gtk_list_store_set(GTK_LIST_STORE(plugin_store), &iter, 0,PLUGIN_STATS, 1,"<b>Plugins:</b>", -1);
+		gtk_list_store_set(GTK_LIST_STORE(plugin_store), &iter, 0,PLUGIN_STATS, 1, _("<b>Plugins:</b>"), -1);
 		for(i=0; i< num_plugins; i++)
 		{
 			if(plugins[i]->pref != NULL && plugins[i]->id&PLUGIN_ID_MARK)
@@ -252,7 +253,8 @@ static void plugin_stats_construct(GtkWidget *container)
 		/**
 		 * new 
 		 */
-		store = gtk_list_store_new(4,G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING,G_TYPE_POINTER);
+		store = gtk_list_store_new(5,G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING,G_TYPE_POINTER,G_TYPE_STRING);
+
 		gtk_tree_view_set_model(GTK_TREE_VIEW(tree),GTK_TREE_MODEL(store));
 		renderer = gtk_cell_renderer_toggle_new();
 		g_object_set_data(G_OBJECT(renderer), "editable", GINT_TO_POINTER(1));
@@ -262,12 +264,16 @@ static void plugin_stats_construct(GtkWidget *container)
 		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(tree), -1,_("Name"), renderer, "text", 1,NULL);
 		renderer = gtk_cell_renderer_text_new();
 		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(tree), -1,_("Function"), renderer, "text", 2,NULL);
+		renderer = gtk_cell_renderer_text_new();
+		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(tree), -1,_("Version"), renderer, "text", 4,NULL);
 		for(i=0;i<num_plugins;i++)
 		{
 			if(plugins[i]->id&PLUGIN_ID_MARK)
 			{
+                gchar *version = g_strdup_printf("%i.%i.%i",plugins[i]->version[0], plugins[i]->version[1],plugins[i]->version[2]);
 				gtk_list_store_append(store, &iter);
-				gtk_list_store_set(store, &iter, 0,TRUE,1, plugins[i]->name,3,(plugins[i]), -1);
+				gtk_list_store_set(store, &iter, 0,TRUE,1, plugins[i]->name,3,(plugins[i]),4,version, -1);
+                g_free(version);
 				if(plugins[i]->get_enabled != NULL)
 				{
 					gtk_list_store_set(store, &iter, 0,plugins[i]->get_enabled(),-1);
@@ -278,20 +284,20 @@ static void plugin_stats_construct(GtkWidget *container)
 						gtk_list_store_set(store, &iter, 2, _("Dummy"),-1);
 						break;
 					case GMPC_PLUGIN_PL_BROWSER:
-						gtk_list_store_set(store, &iter, 2, _("Browser Extention"),-1);
+						gtk_list_store_set(store, &iter, 2, _("Browser Extension"),-1);
 						break;
 					case GMPC_PLUGIN_META_DATA:
 						gtk_list_store_set(store, &iter, 2, _("Metadata Provider"),-1);
 						break;
           case GMPC_PLUGIN_PL_BROWSER|GMPC_PLUGIN_META_DATA:
-						gtk_list_store_set(store, &iter, 2, _("Metadata Provider and Browser Extention"),-1);
+						gtk_list_store_set(store, &iter, 2, _("Metadata Provider and Browser Extension"),-1);
             break;
 					case GMPC_PLUGIN_NO_GUI:
 						gtk_list_store_set(store, &iter, 2, _("Misc."),-1);
 						break;                                                         					
 
 					default:
-						gtk_list_store_set(store, &iter, 2, _("Unkown"),-1);
+						gtk_list_store_set(store, &iter, 2, _("Unknown"),-1);
 						break;
 
 				}
