@@ -494,7 +494,34 @@ q_async_queue_timed_pop_unlocked (QAsyncQueue* queue, GTimeVal *end_time)
 
   return q_async_queue_pop_intern_unlocked (queue, FALSE, end_time);
 }
+/**
+ * q_async_queue_true_length:
+ * @queue: a #QAsyncQueue.
+ * 
+ * Returns the length of the queue, negative values mean waiting
+ * threads, positive values mean available entries in the
+ * @queue. Actually this function returns the number of data items in
+ * the queue minus the number of waiting threads. Thus a return value
+ * of 0 could mean 'n' entries in the queue and 'n' thread waiting.
+ * That can happen due to locking of the queue or due to
+ * scheduling.  
+ *
+ * Return value: the length of the @queue.
+ **/
+gint
+q_async_queue_true_length (QAsyncQueue* queue)
+{
+  gint retval;
 
+  g_return_val_if_fail (queue, 0);
+  g_return_val_if_fail (g_atomic_int_get (&queue->ref_count) > 0, 0);
+
+  g_mutex_lock (queue->mutex);
+  retval = queue->queue->length;
+  g_mutex_unlock (queue->mutex);
+
+  return retval;
+}
 /**
  * q_async_queue_length:
  * @queue: a #QAsyncQueue.
@@ -676,4 +703,27 @@ q_async_queue_remove_data_unlocked (QAsyncQueue* queue, GCompareFunc func, gpoin
 	}
   return NULL;
 }
+
+gboolean
+q_async_queue_has_data (QAsyncQueue *queue, GCompareFunc func, gpointer data)
+{
+	gint length;
+	GList *list = NULL;
+
+	g_return_val_if_fail (queue,FALSE);
+	g_return_val_if_fail (g_atomic_int_get (&queue->ref_count) > 0, FALSE);
+
+	/* get the first */
+	list = g_queue_peek_nth_link(queue->queue, 0);
+	for(length = g_queue_get_length(queue->queue);list;length++)
+	{
+		if(!func(list->data, data))
+		{
+			return TRUE;	
+		}
+		list = list->next;
+	}
+	return FALSE;
+}
+
 #define __Q_ASYNCQUEUE_C__
