@@ -87,7 +87,7 @@ static void url_parse_pls_file(const char *data, int size)
 	}
 	if(songs)
 	{
-		char *string = g_strdup_printf(_("Added %i %s"), songs, (songs > 1)? _("streams"):_("stream"));
+		char *string = g_strdup_printf(_("Added %i %s"), songs, ngettext("stream", "streams", songs));
 		pl3_push_statusbar_message(string);
 		q_free(string);
 	}
@@ -116,7 +116,7 @@ static void url_parse_extm3u_file(const char *data, int size)
 	}
 	if(songs)
 	{
-		char *string = g_strdup_printf(_("Added %i %s"), songs, (songs > 1)? _("streams"):_("stream"));
+		char *string = g_strdup_printf(_("Added %i %s"), songs, ngettext("stream", "streams", songs));
 		pl3_push_statusbar_message(string);
 		q_free(string);
 	}
@@ -161,11 +161,14 @@ static void url_progress_callback(int size, int total,GtkProgressBar *pb)
 }
 static int url_check_binary(char *data, int size)
 {
-	int i=0;
+/*	int i=0;*/
 	int binary = FALSE;
+    /*
 	for(i=0;i < size;i++) {
 		if((unsigned int)data[i] > 127) binary = TRUE;
 	}
+    */
+    binary = !g_utf8_validate(data, size,NULL);
 	if(binary)
 		printf("Binary data found\n");
 	return binary;
@@ -182,12 +185,15 @@ void url_start_real(const gchar *url)
 	 * Setup the Dialog
 	 */
 	GtkWidget *vbox = NULL, *label = NULL, *entry=NULL,*ali = NULL, *progress = NULL;
+	GtkWidget *pl3_win = glade_xml_get_widget(pl3_xml, "pl3_win");
 	GtkWidget *add_button = NULL;
 	GtkWidget *dialog = gtk_dialog_new_with_buttons
 		("Open URL", NULL,
 		 GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
 		 NULL);
+
 	gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
+    gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(pl3_win));
 	/* Add buttons */
 	gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL); /** CANCEL BUTTON */
 	add_button = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_ADD, GTK_RESPONSE_OK); /** ADD BUTTON */
@@ -223,73 +229,67 @@ void url_start_real(const gchar *url)
 
 	gtk_widget_show_all(dialog);
 	gtk_widget_hide(progress);
-	switch(gtk_dialog_run(GTK_DIALOG(dialog)))
+
+	while(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) 
 	{
-		case GTK_RESPONSE_OK:
-			{	
-
-				const gchar *text = gtk_entry_get_text(GTK_ENTRY(entry));
-				/*  In any case, don't download more then 2 kbyte */
-				gmpc_easy_download_struct dld = {NULL, 0, 4096,(ProgressCallback)url_progress_callback, progress};
-				gtk_widget_show(progress);
-				gtk_widget_set_sensitive(dialog, FALSE);
-				if(gmpc_easy_download(text, &dld) && dld.size)
-				{
-					if(url_check_binary(dld.data, dld.size))
-					{
-						printf("Adding url: %s\n", text);
-						mpd_playlist_add(connection, (char *)text);
-						pl3_push_statusbar_message(_("Added 1 stream"));
-					}	
-					/** pls file: */
-					else if(!strncasecmp(dld.data, "[playlist]",10))
-					{
-						printf("Detected a PLS\n");
-						url_parse_pls_file(dld.data, dld.size);
-					}
-					/** Extended M3U file */
-					else if (!strncasecmp(dld.data, "#EXTM3U", 7))
-					{
-						printf("Detected a Extended M3U\n");
-						url_parse_extm3u_file(dld.data, dld.size);
-					}
-					/** Hack to detect most non-extended m3u files */
-					else if (!strncasecmp(dld.data, "http://", 7))
-					{
-						printf("Might be a M3U, or generic list\n");
-						url_parse_extm3u_file(dld.data, dld.size);
-					}
-					/** Assume Binary file */
-					else
-					{
-						printf("Adding url: %s\n", text);
-						mpd_playlist_add(connection, (char *)text);
-
-						pl3_push_statusbar_message(_("Added 1 stream"));
-					}
-					gmpc_easy_download_clean(&dld);
-				}
-				else{
-					/** 
-					 * Failed to contact %s
-					 * Show an error dialog  
-					 */
-					/* Make a copy so we can safely destroy the widget */
-					gchar *url = g_strdup(text);
-					/* Destroy old popup */
-					gtk_widget_destroy(dialog);
-					/* Create info dialog */
-					dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
-						GTK_MESSAGE_INFO,GTK_BUTTONS_CLOSE,
-						_("Failed to contact url: '%s'"), url);
-					q_free(url);
-					/* run it */
-					gtk_dialog_run(GTK_DIALOG(dialog));
-
-				}
-			}		
-		default:
-			break;
+		const gchar *text = gtk_entry_get_text(GTK_ENTRY(entry));
+		/*  In any case, don't download more then 2 kbyte */
+		gmpc_easy_download_struct dld = {NULL, 0, 4096,(ProgressCallback)url_progress_callback, progress};
+		gtk_widget_show(progress);
+		gtk_widget_set_sensitive(dialog, FALSE);
+		if(gmpc_easy_download(text, &dld) && dld.size)
+		{
+			if(url_check_binary(dld.data, dld.size))
+			{
+				debug_printf(DEBUG_INFO,"Adding url: %s\n", text);
+				mpd_playlist_add(connection, (char *)text);
+				pl3_push_statusbar_message(_("Added 1 stream"));
+			}
+			/** pls file: */
+			else if(!strncasecmp(dld.data, "[playlist]",10))
+			{
+				debug_printf(DEBUG_INFO,"Detected a PLS\n");
+				url_parse_pls_file(dld.data, dld.size);
+			}
+			/** Extended M3U file */
+			else if (!strncasecmp(dld.data, "#EXTM3U", 7))
+			{
+				debug_printf(DEBUG_INFO,"Detected a Extended M3U\n");
+				url_parse_extm3u_file(dld.data, dld.size);
+			}
+			/** Hack to detect most non-extended m3u files */
+			else if (!strncasecmp(dld.data, "http://", 7))
+			{
+				debug_printf(DEBUG_INFO,"Might be a M3U, or generic list\n");
+				url_parse_extm3u_file(dld.data, dld.size);
+			}
+			/** Assume Binary file */
+			else
+			{
+				debug_printf(DEBUG_INFO,"Adding url: %s\n", text);
+				mpd_playlist_add(connection, (char *)text);
+				pl3_push_statusbar_message(_("Added 1 stream"));
+			}
+			gmpc_easy_download_clean(&dld);
+			gtk_widget_destroy(dialog);
+			return;
+		} else {
+			/**
+			 * Failed to contact %s
+			 * Show an error dialog
+			 */
+			/* Make a copy so we can safely destroy the widget */
+			gchar *url = g_strdup(text);
+			/* Destroy old popup */
+			gtk_widget_destroy(dialog);
+			/* Create info dialog */
+			dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
+				GTK_MESSAGE_INFO,GTK_BUTTONS_CLOSE,
+				_("Failed to contact url: '%s'"), url);
+			q_free(url);
+			/* run it */
+			gtk_dialog_run(GTK_DIALOG(dialog));
+		}
 	}	
 	gtk_widget_destroy(dialog);
 }
